#!/bin/bash
# ===========================================================================
# actioncmd:
#
# Run the automation rule associated with a script and subject.
# ===========================================================================
mydir=`dirname $0`
myname=`basename $0`
scriptname="$1"
subjectname="$2"

# We need the tools_library here, if not available we don't know
# the alert method or logging methods to use so just write a
# message to the system logger before bombing out.
if [ ! -f ${mydir}/tools_library ];
then
   echo "NO"
   logger "${myname}: Missing ${mydir}/tools_library file"
   exit 1
fi
. ${mydir}/tools_library

# Log a message for what we are starting so any errors are in context.
automation_log_message "Beginning automated recovery of ${subjectname} for ${scriptname}"

# Does an automation rule exist, does the file exist, are we the correct userid etc.
testvar=`${mydir}/is_automated.sh ${scriptname} ${subjectname}`
test_result=$?
if [ "${test_result}." = "2." ]; # error alert already written
then
    exit 1
fi
if [ "${testvar}." = "NO." ];
then
   raise_critical_alert "AUTOMATIONCHK" "${myname}: No active automation rule for ${scriptname} ${subjectname}"
   automation_log_message "No active automation rule for ${scriptname} ${subjectname}"
   exit 1
fi

AUTOMATION_KEY=`echo "${testvar}" | awk {'print $2'}`
AUTOMATION_SCRIPT=`echo "${testvar}" | awk {'print $3'}`

# Call the automation script with all the parameters we were provided with
raise_warning_alert "${AUTOMATION_KEY}" "Automation recovery in progress for ${subjectname}"
${AUTOMATION_SCRIPT} $*
automation_result=$?
if [ "${automation_result}." = "0." ];
then
   cancel_alert "${AUTOMATION_KEY}" "Automation recovery completed for ${subjectname}"
   automation_log_message "Automation recovery completed for ${subjectname}"
else
   raise_critical_alert "${AUTOMATION_KEY}" "${myname}: Failure in automated recovery for ${scriptname} ${subjectname}"
   automation_log_messgage "Failure in automated recovery for ${scriptname} ${subjectname}"
fi

exit 0
