#!/bin/bash

#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.0 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.0 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------

# ---------------------------------------------------
# This sample script will check the sizes of files
# listed in the config file, and alert if any of
# the files are above thresholds defined in the
# config files.
#
# Note: I run this as root on my system as some
# of the directories are not visible to normal
# users.
#
# Will...
#   clear any alerts it previously raised
#   raise new alerts as appropriate
#
# Changes
# 18Dec2003 - added alert or execprog fields to datafile
#             and implemented the ability to run a script
#             rather than raise an alert.
# ---------------------------------------------------


# =========================================================
#    G l o b a l    V a r i a b l e s    N e e d e d
# =========================================================
ALERT_START_PROG="/opt/dickinson/alert_management/scripts/raise_alert.sh"
ALERT_END_PROG="/opt/dickinson/alert_management/scripts/end_alert.sh"

# Get the location we are (and our files are).
myname=$0
scriptname=`basename $0`
mypath=`dirname ${myname}`
filelistfile="${mypath}/jobdata/filesize_check.cfg"
alertcache="${mypath}/jobdata/filesize_check.alert_store"
errorlog="${mypath}/jobdata/filesize_check.errors"
AUTOMATION_DIR="${mypath}/../automation"


# =========================================================
# Helper routines, to try and keep the script coding
# a little more readable.
# =========================================================

# ---------------------------------------------------------
# check_alertprog_binaries:
#
# Check our binaries/scripts actually exist
# ---------------------------------------------------------
check_alertprog_binaries() {
   if [ ! -x ${ALERT_START_PROG} ];
   then
      logger "$0: NOT FOUND->${ALERT_START_PROG}"
      exit 1
   fi
   if [ ! -x ${ALERT_END_PROG} ];
   then
      logger "$0: NOT FOUND->${ALERT_END_PROG}"
      exit 1
   fi
} # check_alertprog_binaries

# ---------------------------------------------------------
#
# clear_old_alerts:
#
# Clear any existing alerts, we will replace them if 
# needed with new ones.
#
# ---------------------------------------------------------
clear_old_alerts() {
   if [ -f ${alertcache} ];
   then
      cat ${alertcache} | while read keyname;
      do
         $ALERT_END_PROG ${keyname} "Alert cleared prior to next check" 2>&1 >> ${errorlog}
      done
      /bin/rm -f ${alertcache}
   fi
} # clear_old_alerts

# ---------------------------------------------------------
#
# raise_alert:
#
# Show the alert as filename, msg then path. This will ensure
# the alert message is meaningfull as otherwise filenames with
# long paths may take up the entire alert message display 
# buffer and truncate off the reason for the error.
#
# ---------------------------------------------------------
raise_alert() {
  keyid=$1
  lfname=$2
  lmsg=$3
  ltype=$4
  smallname=`basename ${lfname}`
  dironly=`dirname ${lfname}`
  $ALERT_START_PROG ${keyid} "${smallname} ${lmsg}, ${dironly}" ${ltype} 2>&1 >> ${errorlog}
  echo "${keyname}" >> ${alertcache}  # save to clear later
} # raise_alert

# ---------------------------------------------------------
#
# do_work:
#
# Actually do all the checks here.
#
# ---------------------------------------------------------
do_work() {
   if [ -f ${filelistfile} ];
   then
      linecount=0
      cat ${filelistfile} | grep -v "#" | while read filename warnlim critlim action;
      do
         linecount=$((${linecount}+1))
         keyname="FILSZCHK_${linecount}"
         if [ -f ${filename} ];
         then
            # expected output format is as below...
            # -rw-r--r--    1 root     root        94808 Jul 25 18:01 /var/log/messages
            testvar=`ls -la ${filename}`
            testsize=`echo "${testvar}" | awk {'print $5'}` 
            if [ ${testsize} -gt ${critlim} ];
            then
               if [ "${action}." = "execprog." ]
               then
                  ${AUTOMATION_DIR}/actioncmd.sh ${scriptname} ${filename} ${testsize}
               else   # must be "alert"
                  raise_alert "${keyname}" "${filename}" "> ${critlim}" "critical"
               fi
            else
               if [ ${testsize} -gt ${warnlim} ];
               then
                  raise_alert "${keyname}" "${filename}" "> ${warnlim}" "warning"
               fi
            fi
         else
            raise_alert "${keyname}" "${filename}" "does not exist" "critical"
         fi
      done
   else
      # Note: The file may actually exist, but if this is run as a user other
      # than root it may not have been able to traverse the directory structure
      # to check the file so the -f cannot find it. We report it as if it is
      # a missing file.
      raise_alert "FILSZCHK" "${filelistfile}" "config file does not exist" "critical"
   fi
} # do_work

# =========================================================
#                   M a i n l i n e
# =========================================================
check_alertprog_binaries
clear_old_alerts
do_work
# Clean up the error log, we don't really want it, it was just to store all
# the newlines from the alert program.
/bin/rm -f ${errorlog}

exit 0
