#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
#
# log_parser_processor.sh:
#
#  This is a sample script to process output piped to it
#  by the log_parse utility.
#
mypath=`dirname $0`
cd ${mypath}

# Only run if we have the alert binaries (scripts)
if [ ! -x ../scripts/raise_alert.sh ];
then
   echo "$0: File ../scripts/raise_alert.sh does not exist"
   exit 1
fi
if [ ! -x ../scripts/end_alert.sh ];
then
   echo "$0: File ../scripts/end_alert.sh does not exist"
   exit 1
fi

# This reads the entire line in which is exactly what we want.
while read dataline;
do
   # Break out the key fields, note two spaces after -d so '\ '=space
   cmdrequest=`echo "$dataline" | cut -d\  -f1`
   cmdkey=`echo "$dataline" | cut -d\  -f2`
   msgline=`echo "$dataline" | cut -d\"  -f2`

   # MID: 16Jan2004 optional hostname may now be passed. If passed it will
   #   have a : appended to it and be prefixed to the message text.
   opthostname=`echo "$dataline" | cut -d\"  -f3`
   if [ "${opthostname}." != "." ];
   then
      opthostname="${opthostname}:"
   fi

   # Process the request.
   case "${cmdrequest}" in
       Error)       # An error message from the program
                    ../scripts/raise_alert.sh PARSER-ERR "${opthostname}${dataline}"
                    ;;
       RAISE-ALERT)
                    ../scripts/raise_alert.sh ${cmdkey} "${opthostname}${msgline}"
                    ;;
       CANCEL-ALERT)
                    ../scripts/end_alert.sh ${cmdkey} "${opthostname}${msgline}"
                    ;;
       USERFUNC-1)
                    # In the sample config I have defined user function 1 for
                    # login failures. A sutable site script could go here, or
                    # you can use user function 1 for something else.
                    ;;
       USERFUNC-2)
                    # Raise an alert plus send an e-mail with error as subject
                    ../scripts/raise_alert.sh ${cmdkey} "${msgline}"
                    echo "." | mail -s "${msgline}" root
                    ;;
       USERFUNC-3)
                    # No user function 3 defined
                    logger "$0: No user function 3 defined"
                    ;;
       USERFUNC-4)
                    # No user function 4 defined
                    logger "$0: No user function 4 defined"
                    ;;
       USERFUNC-5)
                    # No user function 5 defined
                    logger "$0: No user function 5 defined"
                    ;;
       *)
                    # Unknown request, ignore it
                    logger "$0: Unknwon request(${dataline})"
                    ;;
   esac
done

# We will only get here when the pip is closed
exit 0
