#!/bin/bash
#
# rc script for Marks Alert Collector server
#
# note: as this should probably not run under root you should set the prefered
#       user to your sites standard/generic monitoring id.
#
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#  The author, Mark Dickinson retains all rights to this software.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.A
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------

# *****************************************************************************
#        S C R I P T     F U N C T I O N S   -   R E A D M E
# *****************************************************************************
#
# This script works fine for me in my environment, you will probably want to
# customise it for yours.
# To run the script as is, including the sample log parser utilities just
# change the global vlaue setting for "userid" below, which identifies
# which user is to run the application. This SHOULD NOT BE the root user.
# 
# BY DEFAULT both the alert server and the sample log tailing tasks are
# started. If you want to change this to just be the alert server AND you
# never intend to run the sample log tailing task change the default action
# where the 'extra' value is checked for in the parameter checks from 'all'
# to 'only' so the script defaults to only the alert server functions.
# 
# The 'basepath' variable is where you installed the alert server package
# to, this should have been customised bythe install script but you should
# check it.
# 
#                 E N D    O F     R E A D M E

# *****************************************************************************
#                             Globals required.
# These are required globally. They control file locations and actions needed.
# *****************************************************************************
basepath="/opt/dickinson/alert_management"
userid="mark"     # prefered user, SITE SPECIFIC: change this
changeusercmd=""
alertchange="NO"
tailchange="NO"
option="start"
extra=""

# *****************************************************************************
#          Set some global values based on parameters provided
#   Adjust the global default values based on any parameters provided.
# *****************************************************************************
option=$1   # what are we doing, "start", "stop" or "status"
extra=$2    # OPTIONAL. may be "only", "tailtask" or "all"; or not provided
            # only is the action affects only the aleret collector server
			# tailtask is to only affect the log tailing tasks
			# all is to affect both, and is the DEFAULT as shipped

if [ "${extra}." = "." ]; # If extra is not provided set the default here
then
   extra="all"  # change default to only here if you are not going to use the log tailing task
fi

# Check the extra parameter, set globals to reflect the value in it.
case "${extra}" in
  all)   
      alertchange="YES"
      tailchange="YES"
      ;;
  only)   
      alertchange="YES"
      tailchange="NO"
      ;;
  tailtask)   
      alertchange="NO"
      tailchange="YES"
      ;;
  *)  # Error, do nothing 
      ;;
esac

# End of setting global values based on parameters passed



# *****************************************************************************
#                           helper subroutines
# *****************************************************************************

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# check_user_exists
#
# Ensures the userid selected exists on the system. Will optionally allow the
# root userid to be a valid userid if "YES" is passed. This option should
# only be used for shutdown, NEVER for start as these tasks should not run
# under the root userid.
# Will abort the script if the user is not valid.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
check_user_exists() {
   rootallowed=$1       # for shutdowns root may be a valid userid
   localuid="${userid}" # for tests to avoid overwriting userid variable

   # If root is allowed, and we are root, then set localuid to
   # root so all the tests pass.
   if [ "${rootallowed}." = "YES." ];
   then
      testuid=`whoami`
	  if [ "${testuid}." = "root." ];
	  then
         localuid="root"
	  fi
   fi

   if [ "${localuid}." = "." ];
   then
      localuid=`whoami`
   fi
   xx=`grep "${localid}" /etc/passwd`
   if [ "${xx}." = "." ];
   then
      echo "*FATAL* The user ${localuid} does not exist on this system"
      echo "*FATAL* Correct the userid in the init.d script and retry."
      echo "*FATAL* Alert collector server has not been started."
      exit 1
   fi
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# check_alertcmd_exists
#
# Called by tasks that need this command script. It will only return from
# this subroutine if the file exists and is executable.
# Will abort the script if the file does not exist.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
check_alertcmd_exists() {
   if [ ! -x ${basepath}/scripts/start_alertserver.sh ];
   then
      echo "*FATAL* ${basepath}/scripts/start_alertserver.sh NOT FOUND"
      echo "*FATAL* Cannot perform requested operation."
      exit 1
   fi
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# start_collector
#
# start the alert collector task under the correct userid.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
start_collector() {
   echo "Starting the alert collector task..."
   check_alertcmd_exists
   if [ "${userid}." = "." ];
   then
      ${basepath}/scripts/start_alertserver.sh start
   else
      su - ${userid} -c "${basepath}/scripts/start_alertserver.sh start"
   fi
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# stop_collector
#
# stop the alert collector task.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
stop_collector() {
   echo "Stopping the alert collector task..."
   check_alertcmd_exists
   ${basepath}/scripts/start_alertserver.sh stop
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# start_tailtask
#
# If the sample log tailing task exists, start it with this routine.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
start_tailtask() {
   echo "Starting the alert tailing task..."
   if [ -x ${basepath}/samples/monitor_var_log_control.sh ];
   then
      if [ "${userid}." = "." ];
      then
         ${basepath}/samples/monitor_var_log_control.sh start
      else
         su - ${userid} -c "${basepath}/samples/monitor_var_log_control.sh start"
      fi
   else
      echo "${basepath}/samples/monitor_var_log_control.sh not found !"
   fi
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# stop_tailtask
#
# stop the log tailing task.
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
stop_tailtask() {
   echo "Stopping the alert tailing task..."
   if [ -x ${basepath}/samples/monitor_var_log_control.sh ];
   then
       ${basepath}/samples/monitor_var_log_control.sh stop
   else
      echo "${basepath}/samples/monitor_var_log_control.sh not found !"
   fi
}


# *****************************************************************************
#                           M A I N L I N E
#
# The globals we needed were set at the start of the script so we assume all
# is OK here. The option will be "start", "stop", "status" or in error.
# For each valid option we need to see if the actions required are for the
# alert server, the log tailing task, or both; we only perform the actions 
# that have been requested.
# For start, only the prefered user is permitted to do the task.
# For stop the prefered user or root may do the task.
# For status we don't care, let anyone do it.
# *****************************************************************************
case "${option}" in
  start)
         check_user_exists "NO"   # cannot permit root is user is expected to be another
         if [ "${alertchange}." = "YES." ];
         then
            start_collector
         fi
         if [ "${tailchange}." = "YES." ];
         then
            start_tailtask
         fi
         ;;
  stop)
         check_user_exists "YES"  # root may override expected user
         if [ "${tailchange}." = "YES." ];
         then
            stop_tailtask
         fi
         if [ "${alertchange}." = "YES." ];
         then
            stop_collector
         fi
         ;;
  status)
         if [ "${alertchange}." = "YES." ];
         then
            check_alertcmd_exists
			echo "Alert Server status..."
            ${basepath}/scripts/start_alertserver.sh status
            if [ "${tailchange}." = "YES." ];
            then
               echo " "   # pretty formatting only.
            fi
         fi
         if [ "${tailchange}." = "YES." ];
         then
            echo "Tail task status..."
            ${basepath}/samples/monitor_var_log_control.sh status
         fi
         ;;
  *)
         xx=`basename $0`
         echo "Syntax:  ${xx} start | stop | status   [ all | only | tailtask ]"
		 echo "If no optional parameter is provided we work on 'all'."
         ;;
esac

echo "Alert management task operations complete."
# Done
exit 0
