#!/bin/bash

#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------

# ---------------------------------------------------
# This sample script will ping servers to check
# if they are alive, and raise an alert for any
# that fail to respond.
#
# The configuration file is expected to have an
# interface address and a hostname, this allows you
# to check paths through multiple interfaces.
#
# Should be run at 15 minute intervals by the job
# scheduler, or by cron.
#
# Will...
#   clear any alerts it previously raised
#   raise new alerts as appropriate
#
# ---------------------------------------------------
#
ALERT_START_PROG="../scripts/raise_alert.sh"
ALERT_END_PROG="../scripts/end_alert.sh"

# Get the location we are (and our files are).
myname=$0
mypath=`dirname ${myname}`
serverlistfile="${mypath}/server_ping_check.cfg"
alertcache="${mypath}/server_ping_check.alert_store"
errorlog="${mypath}/server_ping_check.errors"

# Check our binaries actually exist
if [ ! -f ${ALERT_START_PROG} ];
then
   logger "$0: NOT FOUND->${ALERT_START_PROG}"
   exit 1
fi
if [ ! -f ${ALERT_END_PROG} ];
then
   logger "$0: NOT FOUND->${ALERT_END_PROG}"
   exit 1
fi

# Clear any existing alerts, we will replace them if 
# needed with new ones.
if [ -f ${alertcache} ];
then
   cat ${alertcache} | while read keyname;
   do
      $ALERT_END_PROG ${keyname} "Alert cleared prior to next check" 2>&1 >> ${errorlog}
   done
   /bin/rm -f ${alertcache}
fi


if [ -f ${serverlistfile} ];
then
   linecount=0
   cat ${serverlistfile} | while read interface servername;
   do
      linecount=$((${linecount}+1))
      keyname="SRVCHK_${linecount}"
      testvar=`ping -I ${interface} -L -c1 ${servername} 2>&1`
      # Check for the unknown host response
      teststr=`echo "${testvar}" | grep -i "unknown host"`
      if [ "${teststr}." != "." ];
      then
         $ALERT_START_PROG ${keyname} "Server ${servername} cannot be resolved" 2>&1 >> ${errorlog}
         echo "${keyname}" >> ${alertcache}  # save to clear later
      fi
      # Check for the no response from server message (unreachable)
      teststr=`echo "${testvar}" | grep -i "Unreachable"`
      if [ "${teststr}." != "." ];
      then
         $ALERT_START_PROG ${keyname} "${servername} is not contactable from ${interface}" 2>&1 >> ${errorlog}
         echo "${keyname}" >> ${alertcache}  # save to clear later
      fi
      # Check for the message that indicates an invalid interface address was used
      teststr=`echo "${testvar}" | grep -i "Cannot assign requested address"`
      if [ "${teststr}." != "." ];
      then
         $ALERT_START_PROG ${keyname} "Network interface ${interface} is not available" 2>&1 >> ${errorlog}
         echo "${keyname}" >> ${alertcache}  # save to clear later
      fi
   done
else
   $ALERT_START_PROG SRVCHK_ERR "Configuration file ${serverlistfile} does not exist" 2>&1 >> ${errorlog}
   echo "SRVCHK_ERR" >> ${alertcache}  # save to clear later
fi

# Clean up the error log, we don't really want it, it was just to store all
# the newlines from the alert program.
/bin/rm -f ${errorlog}

exit 0
