#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------

# ****************************************************************
#
# system_load_average_check.sh
#
# Check system load activity, raise an alert if any thresholds are
# exceeded.
#
# ****************************************************************

ALERT_START_PROG="../scripts/raise_alert.sh"
ALERT_START_PROG="../scripts/end_alert.sh"

# ----------------------------------------------------------------
#                   Set the defaults here
# ----------------------------------------------------------------
keynum=0
myname=`basename $0`
mypath=`dirname $0`
alertcache="${mypath}/${myname}.alert_store"

# ----------------------------------------------------------------
# Some little helper routines to do lots of work and keep the
# mainline looking tidy.
# ----------------------------------------------------------------

# raise alerts for us
raise_critical_alert() {
   key="$1"
   text="$2"
   type="$3"
   if [ "${type}." = "." ];
   then
      type="critical"
   fi
   if [ -x ${ALERT_START_PROG} ];
   then
      ${ALERT_START_PROG} ${key} "${text}" "${type}"
      echo "Raising alert ${key}: ${text}"
      echo "${key}" >> ${alertcache}  # save to clear later
   else
      logger "*FATAL* ${myname}: program not found, ${ALERT_START_PROG}"
      echo "*FATAL* ${myname}: program not found, ${ALERT_START_PROG}"
      exit 1
   fi
} # raise_critical_alert

raise_warning_alert() {
   key="$1"
   text="$2"
   raise_critical_alert "${key}" "${text}" "warning"
} # raise_warning_alert

# ----------------------------------------------------------------
#                      M A I N L I N E
# ----------------------------------------------------------------

# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
# first clear any previous alerts
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
if [ -f ${alertcache} ];
then
   if [ -x ${ALERT_END_PROG} ];
   then
      alertlist=""
      cat ${alertcache} | while read keyname;
      do
         alertlist="${alertlist}${keyname}, "
         ${ALERT_END_PROG} ${keyname} "Alert cleared prior to next check"
      done
      echo "Cleared alerts for ${alertlist} prior to the new check"
   fi
   /bin/rm -f ${alertcache}
fi

# -----------------------------------------------------------------------------
# MID: 2003/06/08 Added the uptime checks to check on system load averages also.
# -----------------------------------------------------------------------------

# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
# 1. Get the uptime string we are to play with
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
a=`uptime`

# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
# 2. The first two fields we want have a trailing comma,
#    remove it.
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
last1min_avg=`echo "${a}" | awk {'print $10'} | cut -d, -f1`
last5min_avg=`echo "${a}" | awk {'print $11'} | cut -d, -f1`
last15min_avg=`echo "${a}" | awk {'print $12'}`

# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
# 3. The values contain 2 numbers after the decimal point,
#    we want to get rid of that so we can compare as if
#    they were whole numbers; so we can check against the
#    decimal part as well.
#    cut -d. -f1,2 leaves the . in, so do in two steps each.
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
b=`echo "${last1min_avg}" | cut -d. -f1`
c=`echo "${last1min_avg}" | cut -d. -f2`
last1min_avg="${b}${c}"
b=`echo "${last5min_avg}" | cut -d. -f1`
c=`echo "${last5min_avg}" | cut -d. -f2`
last5min_avg="${b}${c}"
b=`echo "${last15min_avg}" | cut -d. -f1`
c=`echo "${last15min_avg}" | cut -d. -f2`
last15min_avg="${b}${c}"

# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
# 4. Now we have the values do the checks
#    These values are for my lightly loaded system so I can
#    spot anything creeping up quickly.
#    warning if
#        last1min average > 1.00 (100)
#        last5min average > 0.40 (040)
#        last15min average > 0.10 (010)
#    critical if
#        last1min average > 2.00 (200)
#        last5min average > 1.00 (100)
#        last15min average > 0.50 (050)
# . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
warn1min=100
warn5min=040
warn15min=010
crit1min=200
crit5min=100
crit15min=050

# Report on all three alerts if necessary so alert monitor gets
# a good view of the system.
found=0
if [ ${last15min_avg} -gt ${crit15min} ];
then
   keynum=$((${keynum}+1))
   key="UPTIME_${keynum}"
   raise_critical_alert "${key}" "System 15 minute load avg above ${crit15min}, is ${last15min_avg}"
   found=1
fi
if [ ${last5min_avg} -gt ${crit5min} ];
then
   keynum=$((${keynum}+1))
   key="UPTIME_${keynum}"
   raise_critical_alert "${key}" "System 5 minute load avg above ${crit5min}, is ${last5min_avg}"
   found=1
fi
if [ ${last1min_avg} -gt ${crit1min} ];
then
   keynum=$((${keynum}+1))
   key="UPTIME_${keynum}"
   raise_critical_alert "${key}" "System 1 minute load avg above ${crit1min}, is ${last1min_avg}"
   found=1
fi

# Only need warnings if no criticals were found.
if [ "${found}." = "0." ];
then
   if [ ${last15min_avg} -gt ${warn15min} ];
   then
      keynum=$((${keynum}+1))
      key="UPTIME_${keynum}"
      raise_warning_alert "${key}" "System 15 minute load avg above ${warn15min}, is ${last15min_avg}"
   fi
   if [ ${last5min_avg} -gt ${warn5min} ];
   then
      keynum=$((${keynum}+1))
      key="UPTIME_${keynum}"
      raise_warning_alert "${key}" "System 5 minute load avg above ${warn5min}, is ${last5min_avg}"
   fi
   if [ ${last1min_avg} -gt ${warn1min} ];
   then
      keynum=$((${keynum}+1))
      key="UPTIME_${keynum}"
      raise_warning_alert "${key}" "System 1 minute load avg above ${warn1min}, is ${last1min_avg}"
   fi
fi

exit 0
