#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#  The author, Mark Dickinson retains all rights to this software.
#
#  This version 1.0.0 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This version 1.0.0 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.A
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
# --------------------------------------------------------------------
#
# alert_console.sh
#
# a helper script to run the console interface. It saves
# the user from having to know the configuration information for the
# alert server.
#
# Function: Start the alert console. Uses the values from
#           ../etc/config.txt to locate server and port.
#           Will allow the number and length of lines defined   
#           for a console in config.txt to be overridden.
#
# Syntax: console.sh [ numlines [ linelen ] ]
#
# --------------------------------------------------------------------

# Load the configuration information
myname=$0
mypath=`dirname $0`
if [ -f ${mypath}/../etc/config.txt ];
then
   . ${mypath}/../etc/config.txt
else
   if [ -f ${mypath}/config.txt ];
   then
      . ${mypath}/config.txt
   else
     # We don't have any variables for write_message, so use logger here to
     # write to the syslogd task.
     logger "*FATAL* $0: No configuration files are available in ../etc or ${mypath} !"
     exit 1
   fi
fi

# Check the program exists
if [ ! -x ${INSTALLED_DIR}/Alert_Console_Application.jar ];
then
   echo "*FATAL* Alert_Console_Application.jar not found in ${INSTALLED_DIR}" 
   exit 1
fi

# Execute the program
java -jar ${INSTALLED_DIR}/Alert_Console_Application.jar ${SERVER_NAME} ${PORT}
exit 0
