#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#  The author, Mark Dickinson retains all rights to this software.
#
#  This version 1.0.0 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This version 1.0.0 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.A
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
# --------------------------------------------------------------------
#
# end_alert.sh
#
# a helper script for batch jobs that need to raise cancel an
# existing alert.
#
# Function: cancel an alert message.
#
# --------------------------------------------------------------------

# We must NEVER write output to the screen as this script is
# intended to be run as a child task from a server process. The
# server process will not take kindly to anything trying to write
# to its stdout as the server will normally be a daemon with
# no terminal.
# Use this function insead of any echo commands.
write_message() {
   aa=`date`
   echo "${aa}: $*" >> ${LOGPATH}/${LOG_FILE}
}

# Load the configuration information
myname=$0
mypath=`dirname $0`
if [ -f ${mypath}/../etc/config.txt ];
then
   . ${mypath}/../etc/config.txt
else
   if [ -f ${mypath}/config.txt ];
   then
      . ${mypath}/config.txt
   else
     # We don't have any variables for write_message, so use logger here to
     # write to the syslogd task.
     logger "*FATAL* $0: No configuration files are available in ../etc or ${mypath} !"
     exit 1
   fi
fi

# Is a logpath defined ?, if not use current path
if [ "${LOGPATH}." = "." ];
then
   LOGPATH=${mypath}
fi

# Is a log file defined, if not default one.
if [ "${LOG_FILE}." = "." ];
then
   LOG_FILE="forwarded_alerts.log"
fi

# Get the required parameters
key=$1
text=$2
if [ "${key}." = "." ];
then
   write_message "Syntax: end_alert.sh KEY [\"TEXT MESSAGE\"]" 
   exit 1
fi
if [ "${text}." = "." ];
then
   text="End of alert"
fi

# Check the program exists
if [ ! -x ${INSTALLED_DIR}/${ALERTPROG} ];
then
   write_message "*FATAL* Program ${ALERTPROG} not found in ${INSTALLED_DIR}" 
   exit 1
fi

# Execute the program, note we store the response and write it out on a
# non-zero return code. Origionally I just piped it to the log file but
# when there ws no error (and no message) the script inserted a blank line.
# So, only write program response if an error.
bb=`${INSTALLED_DIR}/${ALERTPROG} ${PORT} ${SERVER_IPADDR} harmless ${key} "${text}"`
result=$?
if [ "${result}." != "0." ];
then
   write_message "${bb}"
   write_message "*ERR: Unable to forward alert ${key}, ${text}"
else
   write_message "Ended  : ${key}, ${text}"
fi

# Always chmod the log file, it's possible that a clear alert is the
# first message for the newday in the log file so we can end up with
# the same ownership/permissions errors that plagued the raise alert
# script when the userid creating the new logfile has a tight umask.
chmod 666 ${LOGPATH}/${LOG_FILE}

# Always exit 0 to avoid problems with server tasks running this script
# as a child process.
exit 0 
