#!/bin/bash
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
#
# start_alertserver.sh
#
# Function: Start/Stop the alert collector task. May be used as
#           the system rc script for this application.
#
# --------------------------------------------------------------------

# --------------------------------------------------------------------
# Load the configuration information
# --------------------------------------------------------------------
myname=$0
mypath=`dirname $0`
if [ -f /opt/dickinson/alert_management/etc/config.txt ];
then
   . /opt/dickinson/alert_management/etc/config.txt
else
   logger "*FATAL* $0: No configuration files are available in ../etc or ${mypath} !"
   exit 1
fi

# --------------------------------------------------------------------
# subproc: start_server
# --------------------------------------------------------------------
start_server() {
    # Check the program exists
    if [ ! -x ${INSTALLED_DIR}/alert_collector_server ];
    then
       logger "$0: *FATAL* Program not found in ${INSTALLED_DIR}" 
       exit 1
    fi

    # Check we are on the correct system for the server task
    aa=`ifconfig -a | grep ${SERVER_IPADDR}`
    if [ "${aa}." = "." ];
    then
       logger "$0: *FATAL* Configured interface does not exist on this server"
       exit 1
    fi

    # Check that it is not already running, use port also in the ps
    # as there may be multiple (prod/dev) copies running.
    aa=`ps -eo args | grep "alert_collector_server ${PORT}" | grep -v "grep"`
    if [ "${aa}." != "." ];
    then
       logger "$0: *WARN* The alert server is already running, no new copy started."
       exit 1
    fi

    # Check the directory paths are correct
    cd ${INSTALLED_DIR}
    if [ ! -d ${LOGPATH} ];
    then
       mkdir ${LOGPATH}
       chmod 775 ${LOGPATH}
    fi
    if [ ! -d ${LOGPATH} ];
    then
       logger "$0: *FATAL* Unable to create directory ${LOGPATH}"
       exit 1
    fi

    # Run from within the logs directory, this will place
    # our log files into it.
    cd ${LOGPATH}

    # if the ipaddr is set to localhost we want to listen on all
    # ports, so reset to nothing.
    if [ "${SERVER_IPADDR}." = "127.0.0.1." ];
    then
       SERVER_IPADDR=""
    fi

    # Clean the evtbuffer file, ensure we start empty
    > evt_buffer.dat
    chmod 664 evt_buffer.dat

    # Execute the program
    ${INSTALLED_DIR}/alert_collector_server ${PORT} ${SERVER_IPADDR}
    aa=$?
    if [ "${aa}." != "0." ];
    then
       echo "Server failed to start, try again later"
       echo "The most common cause is tcp taking a long time to free the port ]"
       netstat -an | grep ${PORT}
    fi
	return
} # end of start_server

# --------------------------------------------------------------------
# subproc: stop_server_OLD
# This is the old way of stopping the server. Keep it around in case
# the clean shutdown has not been installed in this customers site.
# --------------------------------------------------------------------
stop_server_OLD() {
   aa=`ps -eo "pid args" | grep "alert_collector_server ${PORT}" | grep -v "grep"`
   if [ "${aa}." != "." ];
   then
       procpid=`echo "${aa}" | cut -d\  -f1`
       kill ${procpid}
   else
       echo "No alert server is running for port ${PORT}"
   fi
   return
} # end of stop_server_OLD

# --------------------------------------------------------------------
# subproc: stop_server
# The NEW way of stopping the server. Using this allows the server
# to close its sockets in a controlled way.
# --------------------------------------------------------------------
stop_server() {
    # Check the program exists
    if [ ! -x ${INSTALLED_DIR}/clean_shutdown ];
    then
       logger "$0: *WARN* Program not found in ${INSTALLED_DIR}, using OLD way" 
       stop_server_OLD
       exit 0
    fi

	# Check we are on the correct system for the server task, we don't
	# allow a remote server to be stopped from this script.
    aa=`ifconfig -a | grep ${SERVER_IPADDR}`
    if [ "${aa}." = "." ];
    then
       logger "$0: *FATAL* Configured interface does not exist on this server"
       exit 1
    fi

	# All OK, stop the server
    ${INSTALLED_DIR}/clean_shutdown ${PORT} ${SERVER_IPADDR}
	return
} # end of stop_server

# --------------------------------------------------------------------
# subproc: status_server
# --------------------------------------------------------------------
status_server() { # Also used to test syntax of stop to ensure we get right pid
    aa=`ps -eo "pid args" | grep "alert_collector_server ${PORT}" | grep -v "grep"`
    if [ "${aa}." != "." ];
    then
        procpid=`echo "${aa}" | awk '{print $1}'`
    	echo "Alert collector process is running as pid ${procpid}"
#        echo "${aa}"
    	ps -ef | grep ${procpid}
    else
        echo "No alert server is running for port ${PORT}"
    fi
	return
} # end of stop_server


# --------------------------------------------------------------------
# See what we were asked to do, and do it.
# --------------------------------------------------------------------
case "$1" in
   start)
      start_server
      ;;
   stop)
      stop_server
      ;;
   restart)
      stop_server
      # Try to avoid port in use messages as tcp cleans up
	  sleep 30 
      start_server
      ;;
   status)
      status_server
      ;;
   *)
      echo "Syntax: $0 start | stop | restart | status"
      exit 1
esac
exit 0
# --------------------------------------------------------------------
# End of script
# --------------------------------------------------------------------
