/*
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
*/
/*
 * ----------------------------------------------------------------------------
 * shared_utils.c
 *
 * common functions shared between all the applications
 * ----------------------------------------------------------------------------
 */
#include <time.h>

/* =========================================================
 * translate_nl_to_hat:
   ========================================================= */
void translate_nl_to_hat( char *bufarea, int maxlen ) {
   int buf_len, i;
   char *ptr;
   buf_len = strlen(bufarea);
   if (buf_len > maxlen) { buf_len = maxlen; }
   ptr = bufarea;
   for (i = 0; i < buf_len; i++) {
     if (*ptr == '\n') { *ptr = '^'; } 
	 ptr++;
   }
   return;
} /* translate_nl_to_hat */


/* =========================================================
 * translate_hat_to_nl:
   ========================================================= */
void translate_hat_to_nl( char *bufarea, int maxlen ) {
   int buf_len, i;
   char *ptr;
   buf_len = strlen(bufarea);
   if (buf_len > maxlen) { buf_len = maxlen; }
   ptr = bufarea;
   for (i = 0; i < buf_len; i++) {
     if (*ptr == '^') { *ptr = '\n'; } 
	 ptr++;
   }
   return;
} /* translate_hat_to_nl */


/* =========================================================
 * zero_data_field:
   ========================================================= */
void zero_data_field( char *bufarea, int bytes_to_erase ) {
   char *ptr;
   int i;
   ptr = bufarea;
   for (i = 0; i < bytes_to_erase; i++) {
      *ptr = '\0';
      ptr++;
   }
   return;
} /* zero_data_field */


/* =========================================================
 * number_to_string:
 * Will only work on ascii servers.
   ========================================================= */
void number_to_string( char *deststr, int numval, int deststr_size ) {
   int i, numleft, lastdigit, x;
   char *ptr;
   
   ptr = deststr + (deststr_size - 1);
   numleft = numval;
   for (i = 0; i < deststr_size; i++) {
      x = (numleft / 10);
      lastdigit = (numleft - (x * 10));
      numleft = x;
      *ptr = (char)(lastdigit + 48);  /* ascii table only, add hex 30 */
      ptr--;      
   }

   /* done */
   return;
} /* number_to_string */

void number_to_string_with_null( char *deststr, int numval, int deststr_size ) {
   char *ptr;
   int  newlen;
   if (deststr_size <= 1) {
      *deststr = '\0';
   }
   else {
      newlen = deststr_size - 1;
      number_to_string( deststr, numval, newlen );
      ptr = deststr + newlen;
      *ptr = '\0';
   }
   return;
} /* number_to_string_with_null */


/* =========================================================
 * space_fill_string:
   Note... data buffer must be fieldlen+1 so we have
   room for the null terminator, the fieldlen is
   expected to be the buffer area less that 1 byte.
   ========================================================= */
void space_fill_string( char * strptr, int fieldlen ) {
  char *ptr1;
  int  i, j;
  /* find the string terminator */
  i = 0;
  ptr1 = strptr;
  while ((*ptr1 != '\0') && (*ptr1 != '\n') && (i < fieldlen)) {
     i++;
     ptr1++;
  }
  /* found a null ? */
  if ((*ptr1 != '\0') && (*ptr1 != '\n')) {
    /* no, just terminate at length given */
    *ptr1 = '\0';  /* we are where the null should be */
  }
  else {
    /* yes, pad out with spaces */
    for (j = i; j < fieldlen; j++) {
       *ptr1 = ' ';
       ptr1++;
    }
    *ptr1 = '\0';
  }
  return;
} /* space_fill_string */


/* =========================================================
 * set_datestamp:
   ========================================================= */
void set_datestamp( char * strptr ) {
   struct tm time_var;
   struct tm *time_var_ptr;
   time_t time_number;

   time_number = time(0);
   time_var_ptr = (struct tm *)&time_var;
   time_var_ptr = localtime( &time_number );    /* time now as tm */
   /* adjust for the quirks in the returned values */ 
   time_var_ptr->tm_year = time_var_ptr->tm_year + 1900;  /* years returned as count from 1900 */
   time_var_ptr->tm_mon = time_var_ptr->tm_mon + 1;       /* months returned as 0-11 */
   /* MAX_ASRV_DATE_LEN will be defined if the constants for this app are available */
   snprintf( strptr, MAX_ASRV_DATE_LEN+1, "%.4d%.2d%.2d %.2d:%.2d",
           time_var_ptr->tm_year, time_var_ptr->tm_mon, time_var_ptr->tm_mday,
           time_var_ptr->tm_hour, time_var_ptr->tm_min );
   return;
} /* set_datestamp */


/* =========================================================
 * pad_header_fields:
   ========================================================= */
void pad_header_fields( common_header_def * header ) {
   space_fill_string( (char *)&header->productid, ASRV_PRODUCT_ID_LEN );
   space_fill_string( (char *)&header->version, ASRV_PRODUCT_VERSION_LEN );
   space_fill_string( (char *)&header->hostname, MAX_ASRV_HOSTNAME_LEN );
   space_fill_string( (char *)&header->unique_key, MAX_ASRV_KEY_LEN );
   space_fill_string( (char *)&header->response_port, 5 );
   space_fill_string( (char *)&header->source_date_time, MAX_ASRV_DATE_LEN );
   return;
} /* pad_header_fields */
