/*
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
*/

/* alert_console.c :
 *
 * This provides a 'dumb' terminal interface to the alert server to
 * view and acknowledge or delete alerts. It is a simple interface
 * provided as a sample implementation.
 *
 * Will display up to 23 alerts at a time to view more you will have
 * to delete some of the ones displayed (after fixibg them of course).
 *
 * syntax: alert_console [ port ipaddr [onceonly | CMD_EXEC "command string"] ]
 * 
 * if ipaddr is provided you must also provide the port.
 * if ONCEONLY or CMD_EXEC are used all parameters must be provided.
 * ONCEONLY=just display alert list once then exit
 * CMD_EXEC=send command "command string" then exit
 * The default is to start a session refreshing the alert list every
 * 60 seconds, and accepting commands from the user.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include "system_type.h"

#include "../common/console_api_def.h"
/* console_io_data_def console_io_buf;  make global to avoid malloc in timer routines */

char *progname;
unsigned int sec_delay = 60; /* delay for timer alarm, 60 seconds */
int socketid;   /* global as we can't pass it to timer interrupt handler */

int display_lines;
int display_linelen;
int use_blink;
/* ASRV_PRODUCTID_LEN */
char productid[100];

#include "../common/shared_utils.c"

/* Done with debugging
#define DEBUG_ON
*/

#ifdef DEBUG_ON
#include "debuglib.c"
#endif

/* *****************************************************************************
 * init_console_request_buffer:
 *
 * Initialise default values into the console request buffer. The default
 * command will be SL as his is used most often.
   ***************************************************************************** */
void init_console_request_buffer( console_io_data_def * databuf ) {
   strncpy( databuf->msg_header.productid, productid, ASRV_PRODUCT_ID_LEN );
   strncpy( databuf->msg_header.version, application_version, ASRV_PRODUCT_VERSION_LEN );
   gethostname( (char *)&databuf->msg_header.hostname, MAX_ASRV_HOSTNAME_LEN );
   strncpy( databuf->msg_header.unique_key, "N/A", MAX_ASRV_KEY_LEN );
   strncpy( databuf->msg_header.response_port, "0", 6 );  /* not implemeted yet */
   databuf->msg_header.messagetype[0] = 'S';  /* show */
   databuf->msg_header.messagetype[1] = 'L';  /* list */
   databuf->msg_header.expect_reply = 'Y';
   set_datestamp( (char *)&databuf->msg_header.source_date_time );
   zero_data_field( (char *)&databuf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
   translate_nl_to_hat( (char *)&databuf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
   number_to_string_with_null( (char *)&databuf->datastrlen, 0, MAX_ASRV_LEN_STR+1 );
   pad_header_fields( (common_header_def *)databuf );
   databuf->NL = '\n';
} /*init_console_request_buffer */


/* *****************************************************************************
 * connect_to_server:
 *
 * Open and Close of sockets moved to seperate procedures, to support the
 * implementation of an open command to allow users to switch between job
 * scheduler servers without having to restart the application.
 * Use *sockptr to pass back the socket we create.
 * Return 1 if OK, 0 if sockptr needed to be set to null
   ***************************************************************************** */
int connect_to_server(int portnum, char *ipaddr, int *sockptr) {
   int z;
   char   workbuf[4096];
   struct sockaddr_in adr_srvr; /* AF_INET */
   int    len_inet;           /* length */

   /* create server socket address */
   memset(&adr_srvr,0,sizeof adr_srvr);
   adr_srvr.sin_family = AF_INET;
   adr_srvr.sin_port = htons(portnum);
   adr_srvr.sin_addr.s_addr = inet_addr(ipaddr);
#ifdef SOLARIS
   /* Solaris does not have a INADDR_NONE defined
	* but returns -1 for a bad address */
   if (adr_srvr.sin_addr.s_addr == -1 )
#else
   if (adr_srvr.sin_addr.s_addr == INADDR_NONE )
#endif
   {
      printf( "Improperly formatted network address, %s\n", ipaddr );
      return( 0 );  /* failed */
   }
   len_inet = sizeof adr_srvr;

   /* create a tcpip socket to use */
   *sockptr = socket(PF_INET,SOCK_STREAM,0);
   if (*sockptr == -1) {
      printf( "%s: %s\n", strerror(errno), "Unable to create TCPIP socket" );
	  return( 0 ); /* failed */ 
   }

   /* connect to the server */
   z = connect(*sockptr,&adr_srvr,len_inet);
   if (z == -1) {
      snprintf( workbuf, 4095, "connect(2), host %s nothing listening on port %d", ipaddr, portnum );
      printf( "%s: %s\n", strerror(errno), workbuf );
	  return( 0 ); /* failed */
   }

   return( 1 );  /* All OK */
} /* connect_to_server*/


/* *****************************************************************************
 * get_host_and_key_parameters:
 *
 * cmdbuf is expected to be 'command hostname key'.
 *
 * Extract the hostname and the key fields from cmdbuf and return them in the
 * hostfield and keyfield parameters.
   ***************************************************************************** */
int get_host_and_key_parameters( char *cmdbuf, char *hostfield, char *keyfield ) {
   char *ptr, *ptr2, *ptr3;
   int  len;

   ptr = cmdbuf;
   /* skip the command field */
   while ((*ptr != ' ') && (*ptr != '\0') && (*ptr != '\n')) { ptr++; }
   /* skip spaces between command and data */
   while (*ptr == ' ') { ptr++; }

   /* Any hostname ? */
   if ((*ptr == '\0') || (*ptr == '\n')) {
      printf( "ERROR: No hostname provided in command\n" );
      printf( "ERROR: Syntax is:'command hostname key'\n" );
      return( 0 ); /* failed to get parms */
   }
   /* yes, find the end of it */
   ptr2 = ptr;
   while ((*ptr2 != ' ') && (*ptr2 != '\0') && (*ptr2 != '\n')) { ptr2++; }
   len = ptr2 - ptr;
   if (len > MAX_ASRV_HOSTNAME_LEN) { len = MAX_ASRV_HOSTNAME_LEN; }
   strncpy( hostfield, ptr, len );
   ptr3 = hostfield + len;
   *ptr3 = '\0';

   /* see if we have a key field */
   ptr = ptr2;
   while (*ptr == ' ') { ptr++; }
   if ((*ptr == '\0') || (*ptr == '\n')) {
      printf( "ERROR: No alert key provided in command\n" );
      printf( "ERROR: Syntax is:'command hostname key'\n" );
      return( 0 ); /* failed to get parms */
   }
   /* yes, find the end of it */
   ptr2 = ptr;
   while ((*ptr2 != ' ') && (*ptr2 != '\0') && (*ptr2 != '\n')) { ptr2++; }
   len = ptr2 - ptr;
   if (len > MAX_ASRV_KEY_LEN) { len = MAX_ASRV_KEY_LEN; }
   strncpy( keyfield, ptr, len );
   ptr3 = keyfield + len;
   *ptr3 = '\0';

   return( 1 ); /* got them both */
} /* get_host_and_key_parameters */


/* *****************************************************************************
 * dump_buffer:
 *
 * Called when we recieve an invalid response from a server task.
   ***************************************************************************** */
void dump_buffer( char *buf, int buflen ) {
   int i;
   char *ptr;
   ptr = buf;
   for (i = 0; i < buflen; i++) {
     if ((*ptr < ' ') || (*ptr > '~')) { *ptr = '?'; }
	 ptr++;
   }
   *ptr = '\0';
   printf( "Buffer read was '%s'\n", buf );
   return;
} /* dump_buffer */


/* *****************************************************************************
 * process_server_response_alertlist:
 *
 * Specific to prcessing the response from an alert list request. We use this
 * to pretty up the list a little using green reverse/bright etc where needed
 * to highlight each alerts severity.
 *
 * Notes: DF indicates a display buffer with more data to follow in further
 * reads, so we loop reading/displaying while the response remains DF.
   ***************************************************************************** */
void process_server_response_alertlist( console_io_data_def *console_io_buf ) {
   int z, line_counter, bytes_read, linebytesused;
   unsigned int junk;
   char *ptr, *ptr2, *ptr3;
   char workbuffer[ASRV_MAX_CONSOLE_LINE_LEN+1];


   /* Clear the screen */
   printf( "%c%s%c%s", 27, "[H", 27, "[J");

   line_counter = 0;  /* loop trap checker */

   /* Process any response */
   console_io_buf->msg_header.messagetype[0] = 'D';
   console_io_buf->msg_header.messagetype[1] = 'F';
   while (memcmp(console_io_buf->msg_header.messagetype, "DF", 2) == 0)
   {
      z = read(socketid, console_io_buf, sizeof(console_io_data_def));
      /* MID: give the server time to process the
       * request and reply, we are getting some partial data buffers
       * which are causing problems, an immediate reread fails so
       * we delay 1 second between reads of partial buffers and build
       * up the buffer, this works. */
      bytes_read = z;
      while (bytes_read < sizeof(console_io_data_def)) {
         ptr = (char *)console_io_buf;
         ptr = ptr + bytes_read;
         junk = sleep( 1 );  /* 1 second wait */
         z = read(socketid, ptr, (sizeof(console_io_data_def) - bytes_read) );
         if (bytes_read == 0) {
            printf( "Incomplete response from Server, it may have been stopped.\n");
            close(socketid);
            putchar('\n');
            exit(1);
         }
         bytes_read = bytes_read + z;
      }
#ifdef DEBUG_ON
      printf( "DEBUG: Got reply from server, %d bytes read (%d expected)\n", z,
					  sizeof(console_io_data_def) );
#endif
      ptr = (char *)&console_io_buf + bytes_read; /* ensure its null terminated */
      *ptr = '\0';
#ifdef DEBUG_ON
      printf( "DEBUG: Got reply from server, %d bytes read (%d expected)\n", z,
              sizeof(console_io_data_def) );
      printf( "About to dump buffer\n");
      dump_console_buffer_to_stdout( (console_io_data_def *)console_io_buf );
#endif

      /* see if response was an error */
      if (memcmp(console_io_buf->msg_header.messagetype, "DE", 2) == 0)
      {
         translate_hat_to_nl( (char *)&console_io_buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
         printf( "%s\n", console_io_buf->datastr );
      }
      /* else should be a display buffer */
	  else if (
                (memcmp(console_io_buf->msg_header.messagetype, "DF", 2) == 0) ||
                (memcmp(console_io_buf->msg_header.messagetype, "DB", 2) == 0)
              )
      {
         translate_hat_to_nl( (char *)&console_io_buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
         /* Differ from the generic response here, parse out each individual
          * line, and display in the appropriate colour coding.
         printf( "%s", console_io_buf->datastr );  has a trailing newline in reply 
          */
         ptr = (char *)&console_io_buf->datastr;
         while ((*ptr != '\0') && (line_counter < display_lines)) { /* loop until we get to the first null in the buffer */
            ptr2 = ptr;
            while ((*ptr2 != '\n') && (*ptr2 != '\0')) { ptr2++; }
            *ptr2 = '\0';  /* terminate this line */
            /* address messagetype value */
            ptr3 = ptr + (MAX_ASRV_DATE_LEN + 1) + (MAX_ASRV_HOSTNAME_LEN + 1) + (MAX_ASRV_KEY_LEN + 1);
            if (memcmp(ptr3,"AC",2) == 0) { /* critical */
               linebytesused = snprintf( workbuffer, display_linelen, "\x1B[7m %s", ptr ); /* reverse text (7) */
            }
            else if (memcmp(ptr3,"AW",2) == 0) { /* warning */
               linebytesused = snprintf( workbuffer, display_linelen, "\x1B[1m %s", ptr ); /* bright text (1) */
            }
            else if (memcmp(ptr3,"AK",2) == 0) { /* acknowledged */
               if (use_blink != 0) {
                  linebytesused = snprintf( workbuffer, display_linelen, "\x1B[5m %s", ptr ); /* blinking text (5) */
               }
               else {
                  linebytesused = snprintf( workbuffer, display_linelen, " %s", ptr ); 
               }
            }
            else { /* something else, just use normal text */
               linebytesused = snprintf( workbuffer, display_linelen, " %s", ptr ); 
            }
            if (linebytesused >= (ASRV_MAX_CONSOLE_LINE_LEN - 1)) {
               workbuffer[ASRV_MAX_CONSOLE_LINE_LEN - 1] = '\0';
            }
            else {
               while( strlen(workbuffer) < display_linelen ) { strcat( workbuffer, " " ); }
            }
            printf( "%s \x1B[0m \n", workbuffer );
            ptr = ptr2 + 1;  /* skip past null at ptr2 and resume checks */
            line_counter++; /* loop check */
         } /* while ptr != string terminator */
      }
      else {
         printf( "BAD Buffer read...\n" );
         dump_buffer( (char *)&console_io_buf->datastr, z );
         printf( "Unrecognised response from server !\n" );
         printf( "Possible version mismatch, aborting !\n" );
         close(socketid);
         putchar('\n');
         exit( 1 );
      }
   }
   return;
} /* process_server_response_alertlist */

/* *****************************************************************************
 * process_server_response_generic:
 *
 * Read the replied from the server and display the output as determined by the
 * messagetype field of the reply.
 *
 * Notes: DF indicates a display buffer with more data to follow in further
 * reads, so we loop reading/displaying while the response remains DF.
   ***************************************************************************** */
process_server_response_generic( console_io_data_def *console_io_buf ) {
   int z;
   char *ptr;
   unsigned int junk;

   junk = sleep( 1 );  /* 1 second wait */

   /* Clear the screen */
   printf( "%c%s%c%s", 27, "[H", 27, "[J");

   /* Process any response */
   console_io_buf->msg_header.messagetype[0] = 'D';
   console_io_buf->msg_header.messagetype[1] = 'F';
   while (memcmp(console_io_buf->msg_header.messagetype, "DF", 2) == 0)
   {
      z = read(socketid, console_io_buf, sizeof(console_io_data_def));
#ifdef DEBUG_ON
      printf( "DEBUG: Got reply from server\n" );
#endif
      /* if bytes read not what expected then not a valid buffer */
      if (z != sizeof(console_io_data_def)) {
         if (z == 0) {
            printf( "No response from Server, it may have been stopped.\n");
         }
         else {
            printf( "Data recieved was not a server response, check what you connected to !\n");
#ifdef DEBUG_ON
            printf( "Reply len was %d bytes, expected %d bytes\n", z, sizeof(console_io_data_def) );
            dump_console_buffer_to_stdout( (console_io_data_def *)console_io_buf );
#endif
         }
         close(socketid);
         putchar('\n');
         exit( 1 );
      }
	  ptr = (char *)&console_io_buf + z; /* ensure its null terminated */
	  *ptr = '\0';
#ifdef DEBUG_ON
	  dump_console_buffer_to_stdout( (console_io_data_def *)console_io_buf );
#endif
      /* see if response was an error */
      if (memcmp(console_io_buf->msg_header.messagetype, "DE", 2) == 0)
      {
         translate_hat_to_nl( (char *)&console_io_buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
         printf( "%s\n", console_io_buf->datastr );
      }
      /* else should be a display buffer */
	  else if (
                (memcmp(console_io_buf->msg_header.messagetype, "DF", 2) == 0) ||
                (memcmp(console_io_buf->msg_header.messagetype, "DB", 2) == 0) ||
                (memcmp(console_io_buf->msg_header.messagetype, "DS", 2) == 0)
              )
      {
         translate_hat_to_nl( (char *)&console_io_buf->datastr, MAX_ASRV_CONSOLE_DATA_LEN );
         printf( "%s", console_io_buf->datastr ); /* has a trailing newline in reply */
         /* If an alert detail display response add an extra line to the display */
         if (memcmp(console_io_buf->msg_header.messagetype, "DS", 2) == 0) {
            printf( "\nWait, the screen will return automatically to the alert list\n" );
         }
      }
      else {
         printf( "Unrecognised response from server !\n" );
         printf( "Possible version mismatch, aborting !\n" );
         close(socketid);
         putchar('\n');
         exit( 1 );
      }
   }
   return;
} /* process_server_response_generic */


/* *****************************************************************************
 * disconnect_from_server:
 *
 * Use *sockptr to pass the socket to close.
   ***************************************************************************** */
void disconnect_from_server( int sockptr ) {
   /* close the socket and exit */
#ifdef DEBUG_MODE_ON
   printf( "DEBUG: in main (closing socket and ending)\n" );
#endif
   close(sockptr);
   putchar('\n');
   return;
} /* disconnect_from_server */


/* *****************************************************************************
 * write_server_request:
 *
 * write the request buffer to the server task.
   ***************************************************************************** */
void write_server_request( console_io_data_def *console_io_buf ) {
   int z;
   char *ptr;

#ifdef DEBUG_ON
   printf( "DEBUG: in write_server_request (writing %d bytes to socket)\n", sizeof(console_io_data_def) );
#endif

   /* ensure the alert key fields are null terminated and padded */
   ptr = (char *)&console_io_buf->msg_header.unique_key;
   while ( (*ptr != '\0') && (*ptr != '\n') ) { ptr++; }
   if (*ptr == '\n') { *ptr = '\0'; }
   while (strlen(console_io_buf->msg_header.unique_key) < MAX_ASRV_KEY_LEN) {
      strcat(console_io_buf->msg_header.unique_key, " ");
   }
   ptr = (char *)&console_io_buf->msg_header.hostname;
   while ( (*ptr != '\0') && (*ptr != '\n') ) { ptr++; }
   if (*ptr == '\n') { *ptr = '\0'; }
   while (strlen(console_io_buf->msg_header.hostname) < MAX_ASRV_HOSTNAME_LEN) {
      strcat(console_io_buf->msg_header.hostname, " ");
   }

   /* write the message */
   z = write( socketid, console_io_buf, sizeof(console_io_data_def) );
   if (z == -1) {
      printf( "Unable to contact server, aborting\n");
      close(socketid);
      putchar('\n');
      exit( 1 );
   }
} /* write_server_request */


/* *****************************************************************************
 * delete_alerts_displayed:
 *
 * build/format a request buffer to delete all alerts up to the key provided
 * by the user. If cmdbuf is NULL we delete all 23 alerts displayed.
 * Note: no reply is expected from the server for this.
 * return 0 if a syntax error so a needless refresh is not sent.
 * return 1 if the request was sent, so a refresh is needed.
   ***************************************************************************** */
int delete_alerts_displayed( char *cmdbuf ) {
   /* initialise a refresh buffer request */
   console_io_data_def console_io_buf;
   init_console_request_buffer( (console_io_data_def *)&console_io_buf );

   /* if not NULL user has provided the end point */
   if (cmdbuf != NULL) {
      /* get the user provided parameters */
      if (get_host_and_key_parameters( cmdbuf,
             (char *)&console_io_buf.msg_header.hostname,
             (char *)&console_io_buf.msg_header.unique_key ) != 1 ) {
         return( 0 );
      }
   }
   /* else use the one we have saved */
   else {
      printf( "ka command has not yet been implemented, use du instead\n" );
      return( 0 );
   }

   /* set the command flag values */
   console_io_buf.msg_header.messagetype[0] = 'K';
   console_io_buf.msg_header.messagetype[1] = 'K';

   console_io_buf.msg_header.expect_reply = 'N'; /* we want no reply */
   write_server_request( &console_io_buf );

   return( 1 );
} /* delete_alerts_displayed */


/* *****************************************************************************
 * delete_alert:
 *
 * build/format an alert delete request buffer.
 * Note: no reply is expected from the server for this.
 * return 0 if a syntax error so a needless refresh is not sent.
 * return 1 if the request was sent, so a refresh is needed.
   ***************************************************************************** */
int delete_alert( char *cmdbuf ) {
   /* initialise a refresh buffer request */
   console_io_data_def console_io_buf;
   init_console_request_buffer( (console_io_data_def *)&console_io_buf );

   /* get the user provided parameters */
   if (get_host_and_key_parameters( cmdbuf,
          (char *)&console_io_buf.msg_header.hostname,
          (char *)&console_io_buf.msg_header.unique_key ) != 1 ) {
      return( 0 );
   }

   /* set the command flag values */
   console_io_buf.msg_header.messagetype[0] = 'A';
   console_io_buf.msg_header.messagetype[1] = 'D';

   console_io_buf.msg_header.expect_reply = 'N'; /* we want no reply */
   write_server_request( &console_io_buf );

   return( 1 );
} /* delete_alert */


/* *****************************************************************************
 * acknowledge_alert:
 *
 * build/format an alert acnowledge request buffer.
 * Note: no reply is expected from the server for this.
 * return 0 if a syntax error so a needless refresh is not sent.
 * return 1 if the request was sent, so a refresh is needed.
   ***************************************************************************** */
int acknowledge_alert( char *cmdbuf ) {
   /* initialise a refresh buffer request */
   console_io_data_def console_io_buf;
   init_console_request_buffer( (console_io_data_def *)&console_io_buf );

   /* get the user provided parameters */
   if (get_host_and_key_parameters( cmdbuf,
          (char *)&console_io_buf.msg_header.hostname,
          (char *)&console_io_buf.msg_header.unique_key ) != 1 ) {
      return( 0 );
   }

   /* set the command flag values */
   console_io_buf.msg_header.messagetype[0] = 'A';
   console_io_buf.msg_header.messagetype[1] = 'C';

   console_io_buf.msg_header.expect_reply = 'N'; /* no reply wanted from server */
   write_server_request( &console_io_buf );

   return( 1 );
} /* acknowledge_alert */


/* *****************************************************************************
 * show_alert_detail:
 *
 * build/format an request buffer to request detailed information on an alert.
 * Note: a reply IS expeced from the server.
   ***************************************************************************** */
void show_alert_detail( char *cmdbuf ) {
   /* initialise a refresh buffer request */
   console_io_data_def console_io_buf;
   init_console_request_buffer( (console_io_data_def *)&console_io_buf );

   /* get the user provided parameters */
   if (get_host_and_key_parameters( cmdbuf,
          (char *)&console_io_buf.msg_header.hostname,
          (char *)&console_io_buf.msg_header.unique_key ) != 1 ) {
      return;
   }

   /* set the command flag values */
   console_io_buf.msg_header.messagetype[0] = 'S';
   console_io_buf.msg_header.messagetype[1] = 'D';

   console_io_buf.msg_header.expect_reply = 'Y'; /* a reply from the server is required */
   write_server_request( &console_io_buf );
   process_server_response_generic( &console_io_buf );

   return;
} /* show_alert_detail */


/* *****************************************************************************
 * refresh_alert_list:
 *
 * build/format an request buffer to request a list of alerts from the server.
 * We are using the default CONSOLE application name so we will get back 23
 * alerts at most.
 * Note: a reply IS expeced from the server.
   ***************************************************************************** */
void refresh_alert_list( void ) {
   int z;
   char *ptr;

   /* initialise a refresh buffer request */
   console_io_data_def console_io_buf;
   init_console_request_buffer( (console_io_data_def *)&console_io_buf );
   /* the default init was for a SL request so we can send now */

   /* send it */
   console_io_buf.msg_header.expect_reply = 'Y'; /* a reply from the server is required */
   write_server_request( &console_io_buf );
   /* manage any response displays */
   process_server_response_alertlist( &console_io_buf );

   return;
} /* refresh_alert_list */


/* *****************************************************************************
 * command_process:
 *
 * Determine what the users input command was and process it.
   ***************************************************************************** */
int command_process( char * cmdbuf ) {
   char *ptr;

   /* was it an empty string ?, if strlen == 1 then it was only a newline [enter] */
   if (strlen(cmdbuf) == 1) {
      printf( "Dont be impatient, next refresh will be in less than 60 seconds\n" );
   }
   /* was it a quit request, we return 0 for that */
   else if (memcmp( cmdbuf, "q", 1 ) == 0) {
      printf( "Got QUIT\n" );
      return( 0 );
   }
   /* Was it a delete all */
   else if (memcmp( cmdbuf, "du", 2 ) == 0) /* delete upto request */
   {
	  if ( delete_alerts_displayed( cmdbuf ) == 1 ) {
         refresh_alert_list();
      }
   }
   else if (memcmp( cmdbuf, "ka", 2 ) == 0) /* kill all */
   {
	  delete_alerts_displayed( NULL );
      /* no refresh, let the command not implemented be displayed */
   }
   /* Was it a delete request */
   else if (memcmp( cmdbuf, "d", 1 ) == 0) /* delete request */
   {
	  if (delete_alert( cmdbuf ) == 1) {
         refresh_alert_list();
      }
   }
   /* Was it a acknowledge request */
   else if (memcmp( cmdbuf, "a", 1 ) == 0) /* acknowledge request */
   {
	  if (acknowledge_alert( cmdbuf ) == 1) {
         refresh_alert_list();
      }
   }
   /* Was it a detailed info (show) request */
   else if (memcmp( cmdbuf, "s", 1 ) == 0) /* show request */
   {
	  show_alert_detail( cmdbuf );
   }
   /* Was it a refresh command */
   else if ( (memcmp( cmdbuf, "refresh", 7 ) == 0) || (memcmp(cmdbuf,"rxx",3) == 0) )
   {
      refresh_alert_list();
   }
   /* Was it a help request */
   else if ( (memcmp( cmdbuf, "h", 1 ) == 0) ||
             (memcmp( cmdbuf, "?", 1 ) == 0) ) {
      printf( "alert_console: Part of the centralised alert console application.\nCopyright (C) 2003  Mark Dickinson\n\n" );
      printf( "Version %s is a free release, it comes with no warantee of\n", application_version );
      printf( "suitability for your site, use at your own risk.\n\n" );
      printf( "alert_console commands available...\n" );
      printf( "          show details - s hostname key\n" );
      printf( "     acknowledge alert - a hostname key\n" );
      printf( "          delete alert - d hostname key\n" );
      printf( "      delete all until - du hostname key\n" );
      printf( "       refresh display - refresh [the full word] (or wait for 60 sec)\n" );
      printf( "          q(uit), exit - q\n" );
	  printf( "\nMessage display colours...\n" );
	  printf( " reverse text  = unacknowledged alert\n" );
      if (use_blink != 0) {
          printf( " blinking text = acknowledged alert, being worked on\n" );
          printf( " normal text   = info only\n" );
      }
	  else {
          printf( " normal text   = acknowledged alert, or an info only\n" );
          printf( "(both in normal text as you have disabled the blink option)\n" );
      }
      /* Note: backdoor refresh=rxx */
   }
   else {
      printf ("WHAT? - need 'd hostname key', 'a hostname key', 's hostname key',\ndu hostname key, ka or 'q'(quit)\n" );
   }

   return( 1 ); /* keep going */
} /* command_process */


/* *****************************************************************************
 * timer_expired:
 *
 * Called when our refresh timer expires, we need to redraw the screen the
 * user is viewing with an updated list of alerts.
   ***************************************************************************** */
int timer_expired_handler( void ) {
#ifdef DEBUG_ON
   printf( "DEBUG: Timer expired\n" );
#endif

   /* On timer expiry we refresh the alert list display */
   refresh_alert_list();

   /* when we have done ALL the required processing
	* start another timer. */
   alarm( sec_delay );

   return( 0 );
} /* timer_expired_handler */


/* *****************************************************************************
 * main:
 *
 * read & check startup parameters, if ok...
 *    connect to server
 *    if interactive,
 *       start timer for refreshes and loop getting responses from user until
 *       user enters "q". Timer will refresh screen with updated alerts
 *       periodically.
 *    if non-interactive
 *       exec required command
 *    disconnect from server & exit.
   ***************************************************************************** */
int main(int argc, char **argv) {
   int z, j, stop_now, once_only, cmdexec_only, command_result;
   char *zptr, *ptr;
   char *srvr_addr = NULL;
   int  srvr_port;
   char   workbuf[4096];
   char   stored_command[255];
   unsigned int last_delay_left, junk;
   int timer_expired_handler();
/*
   const struct sigaction new_timer_action;
   const struct sigaction old_timer_action;
*/

   progname = argv[0]; /* store our name */

   /* --- check for overrides of the defaults --- */
   /* if we have been provided with a port number use it */
   if (argc >= 2 ) {
      srvr_port = atoi(argv[1]);
   }
   else {
      srvr_port = 9003; /* alert server default port is 9003 */
   }
   /* if we have been provided with a server address
      then we will use it. If not use localhost 127.0.0.1 */
   if (argc >= 3 ) {
      srvr_addr = (char *)argv[2];
   }
   else {
      srvr_addr = "127.0.0.1";
   }
   /* There may be optional flags, check here */
   once_only = 0;     /* default is don't just poll once */
   cmdexec_only = 0;  /* default is no command to issue */
   display_lines = 24;
   display_linelen = 100;
   use_blink = 1; 
   if (argc >= 4) {
      for (j = 3; j < argc; j++) {
          /* Check for the optional once only flag */
          if ( (memcmp(argv[j],"onceonly",8) == 0) ||
               (memcmp(argv[j],"ONCEONLY",8) == 0) ) {
             once_only = 1;
    	  }
    	  else if (memcmp(argv[j],"CMD_EXEC",8) == 0) {
              if (argc >= (j+2) ) { /* ensure a command is present */
                 cmdexec_only = 1; /* exec the command and exit */
                 j++;
                 strncpy( stored_command, argv[j], 254 );
                 stored_command[254] = '\0';
              }
              else {
                 printf( "Missing parameters for a cmdexec, refer to the manual for command syntax\n" );
                 exit( 1 );
              }
    	  }
		  else if ( (memcmp(argv[j],"linelen=",8) == 0) ||
                    (memcmp(argv[j],"LINELEN=",8) == 0) ) {
             ptr = (char *)argv[j];
             ptr = ptr + 8;
             display_linelen = atoi(ptr);
    	  }
		  else if ( (memcmp(argv[j],"numlines=",9) == 0) ||
                    (memcmp(argv[j],"NUMLINES=",9) == 0) ) {
             ptr = (char *)argv[j];
             ptr = ptr + 9;
             display_lines = atoi(ptr);
    	  }
		  else if ( (memcmp(argv[j],"blinkoff",8) == 0) ||
                    (memcmp(argv[j],"BLINKOFF",8) == 0) ) {
             use_blink = 0;
    	  }
          else {
             printf( "Unknown command flag: %s, refer to the manual for command syntax\n", argv[j] );
             exit( 1 );
          }
      } /* for j */
   } /* if argc > 4 */

   /* Check the numeric input */
   if ((display_linelen == 0) || (display_linelen > ASRV_MAX_CONSOLE_LINE_LEN)) {
     printf( "Invalid LINELEN= value, using 90\n" );
     display_linelen = 90;
   }
   if ((display_lines == 0) || (display_lines > 99)) {
     printf( "Invalid NUMLINES= value, using 24\n" );
     display_lines = 24;
   }

   /* build out product id */
   strcpy( productid, "CONSOLE" );
   ptr = (char *)&productid + 7;
   number_to_string( ptr, display_lines, 2 );
   strcat( ptr, "_" );
   ptr = (char *)&productid + strlen(productid);
   number_to_string_with_null( ptr, display_linelen, 4 );

   /* Display the banner only if we are running an interactive session */
   if ((cmdexec_only == 0) && (once_only == 0)) {
      /* Display the banner */
      printf( "%c%s%c%s", 27, "[H", 27, "[J");
      printf( "Alert Console, part of the alert message tools by Mark Dickinson\n");
      printf( "(C)Mark Dickinson, 2003. All rights reserved.\n");
      printf( "This software comes with ABSOLUTELY NO WARRANTY, you\n");
      printf( "use it at your own risk. The author is not liable\n");
      printf( "for any damages percieved or otherwise caused by use of\n");
      printf( "this application.\nThis software is released under the\n");
      printf( "GPL license; for details see the copy of the GPL license\n");
      printf( "that was shipped with this file.");
      printf( "\n\nUsing %d lines of %d line length, refer to the\n",
              display_lines, display_linelen );
      printf( "manual if you wish to change these defaults.");
      printf( "\n\nConnection to the alert server will be attempted in 5 seconds.\n");
      junk = sleep( 5 );
   }

   /* Connect to the job scheduler server */
   if ((command_result = connect_to_server( srvr_port, srvr_addr, &socketid )) == 0) {
      printf( "Unable to connect to server.\n" );
      exit( 1 );
   }

   /* ----------------------------------------------
	* initialisation completed, now we need to do
	* all the hard work
    * ---------------------------------------------- */
   /* we need to do two things here,
	* 1) autorefresh from the server every n seconds
	* 2) accept keyboard commands
	*/

   /* Are we executing one command then exiting ? */
   if (cmdexec_only == 1) { /* issue the command and exit */
      command_result = command_process( (char *)argv[4] );
      once_only = 1; /* use to avoid the loop */
   }
   /* else we need the alert display at least once */
   else {  /* else we need the alert display at least once */
      /* display the initial display screen */
      refresh_alert_list();
   }

   /* Are we runnining interactively (request more than once) */
   if (once_only == 0) { /* if we are to loop expecting user input */
      /* we need to do two things here,
       * 1) autorefresh from the server every n seconds
       * 2) accept keyboard commands
       */
      /* Set up a timer */
      sec_delay = 60;
      signal( SIGALRM, timer_expired_handler );
      alarm( sec_delay );
      /* read commands until command is q (quit) */
      stop_now = 0;
      while (stop_now == 0) {
         zptr = fgets(workbuf,sizeof workbuf - 1,stdin);
         printf( "\n" );
         if (zptr != NULL) {
            command_result = command_process( (char *)&workbuf );
            if (command_result == 0) { stop_now = 1; }
         }
      }      /* while */
   } /* if refresh required more than once */

   /* ----------------------------------------------
	* We have finished.
    * close the socket and exit
    * ---------------------------------------------- */
   /* close the socket and exit */
   disconnect_from_server( socketid );
   return(0);
} /* main */
