package console.client;

/**
 * This is a customised MouseAdapter class used to manage the
 * popup menus we need, and to send commands to the remote alert
 * application based on the menu items selected.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.Vector;
import java.io.*;

import console.client.SocketObject;
import console.client.Constants;

/* ------------------------------------------------------------------------------
   Alert_Console_List_MouseAdapter:
   Handles all right click mouse events performed on out JList.
   ------------------------------------------------------------------------------ */
public class Alert_Console_List_MouseAdapter extends MouseAdapter implements Constants {

    /**
    Used to create a popup menu for the main alert list
    and manage the events/commands requested from the popup menu.
    */

    JPopupMenu list_popupMenu;
    JList myDataList;
    SocketObject ipSocket;

    // Define the listener for popup menu events
    class popup_handler_listener implements ActionListener {
       /**
        * The ActionListener class for the popup menu.
        *
        * When a popup menu item is selected it will parse the
        * menu line selected at the time the popup menu was invoked
        * to obtain the information it needs to identify which alert
        * the command is to be issued against.
        *
        * It will then sends the appropriate command to the server
        * application based on the menu entry selected from the
        * popup list.
        */
       public void actionPerformed(ActionEvent e) {
          // Only do anything if a line has been selected.
          if (myDataList.getSelectedIndex() != -1)
          {
              // Mucking about to get the text string
              Object strdetails = myDataList.getSelectedValue();
              String dataTextLine = ((ListItem)strdetails).getValue();
              // Extract the hostname and key so we can use them to
              // issue commands against the message.
              String hostname = dataTextLine.substring(15,30); // hostnamelen is 15
              String msgkey = dataTextLine.substring(31,45);   // keylen is 14
              String actionCode = dataTextLine.substring(46,48);
              if (e.getActionCommand() == "Acknowledge Alert") {
                 if ( (actionCode.equals("AC")) || (actionCode.equals("AW")) ) {  // only ack unacknkowledged alerts
                    ipSocket.writeBuffer( 
                        API_CMD_ACKNOWLEDGE_1+hostname+'\0'+msgkey+API_CMD_ACKNOWLEDGE_2
                    );
                 }
                 ipSocket.writeBuffer( 
                     API_CMD_REFRESHLIST
                 );
              }
              else if (e.getActionCommand() == "Delete Alert") {
                 ipSocket.writeBuffer( 
                     API_CMD_DELETE_1+hostname+'\0'+msgkey+API_CMD_DELETE_2
                 );
                 ipSocket.writeBuffer( 
                     API_CMD_REFRESHLIST
                 );
              }
              else if (e.getActionCommand() == "Delete All Until Alert") {
                 ipSocket.writeBuffer( 
                     API_CMD_DELETEUNTIL_1+hostname+'\0'+msgkey+API_CMD_DELETEUNTIL_2
                 );
                 ipSocket.writeBuffer( 
                     API_CMD_REFRESHLIST
                 );
              }
              else if (e.getActionCommand() == "Show Details") {
                 if (!hostname.equals("---------------")) {  // no details on system info msgs
                    ipSocket.writeBuffer( 
                        API_CMD_SHOWDETAILS_1+hostname+'\0'+msgkey+API_CMD_SHOWDETAILS_2
                    );
                 }
              }
              else {
                 // Unknown, log for debugging.
                 System.out.println("field is "+e.getActionCommand());
              }
          }  // of != -1
       }
     }  /* popup_handler_listener */


   public Alert_Console_List_MouseAdapter(JList dataList, SocketObject sockid) {
      /**
      Constrctor class to create the popup menu list.
      Expects to be passed the JList this popup menu is being used on
      and the SocketObject created by the main environment so it
      knows which socket to send commands down.
      */
      myDataList = dataList;
      ipSocket = sockid;
      // A menu for right mouse clicks in the list
      list_popupMenu = new JPopupMenu();
      popup_handler_listener popup_handler = new popup_handler_listener();
      // The list popupmenu
      JMenuItem popupItem1 = new JMenuItem( "Show Details", KeyEvent.VK_S );
      popupItem1.addActionListener(popup_handler);
      list_popupMenu.add( popupItem1 );
      list_popupMenu.addSeparator();
      JMenuItem popupItem2 = new JMenuItem( "Acknowledge Alert", KeyEvent.VK_A );
      popupItem2.addActionListener(popup_handler);
      list_popupMenu.add( popupItem2 );
      JMenuItem popupItem3 = new JMenuItem( "Delete Alert", KeyEvent.VK_D );
      popupItem3.addActionListener(popup_handler);
      list_popupMenu.add( popupItem3 );
      list_popupMenu.addSeparator();
      JMenuItem popupItem4 = new JMenuItem( "Delete All Until Alert", KeyEvent.VK_U );
      popupItem4.addActionListener(popup_handler);
      list_popupMenu.add( popupItem4 );
   }

   public void mousePressed( MouseEvent e ) { checkForTriggerEvent( e ); }
   public void mouseReleased( MouseEvent e ) { checkForTriggerEvent( e ); }
   private void checkForTriggerEvent( MouseEvent e ) {
      if ( e.isPopupTrigger() ) {
         // Only show if a list item is selected
         if (myDataList.getSelectedIndex() != -1)
         {
            list_popupMenu.show( e.getComponent(), e.getX(), e.getY() );
         }
      }
   }
} /* Alert_Console_List_MouseAdapter */
