package console.client;

import java.net.*;
import java.io.*;
import javax.swing.DefaultListModel;
import javax.swing.SwingUtilities;
import java.awt.Color;
// import console.client.SocketReadThread;
import console.client.ListItem;
import console.client.DisplayDetailsWindow;

/**
 * This class creates and manages the new TCP-IP connection socket to
 * the server application.
 *
 * To replace the original CreateNewSocket and SocketIdentifer classes
 * and allow a way of reconnecting to hosts that go down and back up.
 */

public class SocketObject {
    /**
     * Create the tcpip sockets needed for this java application
     * to communicate withthe alert server.   
     */

    protected Socket s;
    protected BufferedReader in;
    protected PrintWriter out;
	protected int portnum;
	protected String hostname;
	protected Thread threadtask;
	public boolean isConnected;
	public DefaultListModel myListModel;
	protected boolean disConnectQueued;
	protected String ServerDataLine;  // for partial line reads
	protected String ServerData;      // to store the full buffer

    /*
    Open a new socket connection to a host.
    If this fails we set isConnected to false so it can be retried later.
    */

	// Constructor
    public SocketObject(
                    String hostname,
                    int portnum,
                    DefaultListModel parentmodel) {
       /**
        * This created a reader thread that will pass the data
        * it recieves to the parent table provided for table refreshes. A data 
        * class of SocketIdentifier is returned as a result of ConnectSock.
        * 
        * It is expected to be passed the remote host name, the remote host
        * port number, and the list model being used by the JList created in
        * main. The JList model is required as it must be passed to the socket
        * listening thread so the thread knows where and how to write data it
        * recieves from the server connection.
        */
       isConnected = false;      // default is not connected yet
       disConnectQueued = false; // no disconnect queued yet
	   this.hostname = hostname;
	   this.portnum = portnum;
	   this.myListModel = parentmodel;
	   this.in = null;
	   this.out = null;
	   threadtask = null;
       isConnected = reConnect();
    }

    private boolean reConnect() {
       boolean connectStatus;
       connectStatus = false; // default is failure
       ServerData = "";
       try {
          s = new Socket(hostname,portnum);
          out = new PrintWriter(s.getOutputStream(),true);
          in = new BufferedReader(new InputStreamReader(s.getInputStream()));
          // added the thread task creation here rather than in the mainline.
          if (threadtask == null) {
             threadtask = new Thread() {
                    public void run() {
                       /**
                        * Loops repeatedly while the BufferedReader connection remains live.
                        * Reads the data we are recieving from the remote server associated
                        * with this thread and determines how to process them.
                        *
                        * If a normal alert list refresh the response is processed within
                        * this class by process_CONSOLE_response which will refresh the alert
                        * list in the main environment.
                        *
                        * If the buffer read from the stream is a detailed display request
                        * then the DisplayDetailsWindow class is requested to handle it.
                        */
                       String productId;
                       String dataPacketType;
                       String dataBuffLenStr;
                       int dataBuffLen;
                       try {
                           while ( (ServerDataLine = in.readLine()) != null ) {
                              ServerData = ServerData + ServerDataLine; // buffer the bits up
                              if (ServerData.length() > 84) {   // must be large enough to get a header field
                                 // Play with extracting the fields now
                                 // Only process a buffer in the CONSOLE format.
                                 productId = ServerData.substring(0,7);
                                 if (productId.equals("CONSOLE")) {
                                    // Pass only the data content of the buffer, we can skip the
                                    // length field as well.
                                    // skip header fields...
                                    //   product(15), version(11), hostname(16), key(15), port(6),
                                    //   type(2), replyexpected(1), datestamp(15) + null = 81.

									// len field is 4 + null added to that, we need the len
                                    // get the length of the data buffer
                                    dataBuffLenStr = ServerData.substring( 81, 85 );
                                    // only do any work if we have a full buffer (81 byte header len, +4 byte len +
                                    // null, + data buffer len
                                    dataBuffLen = StringToInt( dataBuffLenStr );
                                    dataBuffLen = dataBuffLen + 86; // add header block len + len field len
                                    if (ServerData.length() > dataBuffLen) {
                                       // Ensure we are an alert list display buffer for normal
                                       // processing, if a DE (error) or DS (show details) then we
                                       // need special processing.
                                       dataPacketType = ServerData.substring( 63, 65 );
                                       if (dataPacketType.equals("DB") || dataPacketType.equals("DF")) {
                                          // FATAL FLAW. will not work if we get multiple
                                          // buffers (continuation replies) to process as we
                                          // would clear in the middle of them. We know we can manage the
                                          // full 50 possible as of 2003/07/10; but recheck this every
                                          // time the response format in the server changes.
                                          //
                                          // We assume we are refreshing the list here, so
                                          // clear any existing entries.
                                          myListModel.clear();
                                          // note: start at position 86
                                          process_CONSOLE_response( ServerData.substring( 86, ServerData.length() ) );
                                          ServerData = "";
                                       }
                                       else if (dataPacketType.equals("DS")) {
                                          DisplayDetailsWindow tempWindow =
                                            new DisplayDetailsWindow( ServerData.substring( 86, ServerData.length() ) );
                                          ServerData = "";
                                       }
                                       else if (dataPacketType.equals("DE")) {
                                          System.out.println( "DEBUG: type is error display response; not implemented" );
                                          ServerData = "";
                                       }
                                       else {
                                          System.out.println( "DEBUG: dataPacketType = '"+dataPacketType+"', no handling for this type yet" );
                                          ServerData = "";
                                       }
                                    } // else incomplete buffer, wait until it fills
                                 }
                                 else {   // not a console response buffer
                                   System.out.println( "ERROR: Unknown response type ("+productId+") ignored" );
                                   ServerData = "";
                                 }
							  } // if a full buffer available
                              yield();
                           } // while != null
                           // if null, we have been diconnected
                           disConnectQueued = true;
                           System.out.println( "Session closed by remote server" );
                           ServerData = "";
                       }
                       catch (IOException e) {
                           System.out.println( "Error in read thread, starting to disconnect" );
                           disConnectQueued = true;
                           ServerData = "";
                       }
                       catch (IndexOutOfBoundsException e2) {
                           System.out.println( "IndexOutOfBounds, io buffer left is..." );
                           System.out.println( ServerData );
                           ServerData = "";
                       }
                    } // Run method
             };  // threadtask =
          }
          threadtask.start();
          connectStatus = true;
       }
       catch (IOException e) {
          connectStatus = false;
       }
       disConnectQueued = false;
       return( connectStatus );
	} // reConnect

    public void disConnect() {
       try {
		  threadtask.stop();
          if (s != null) { s.close(); s = null; }
       }
       catch (IOException e) {
          s = null;
       }
       isConnected = false;
       in = null;
       out = null;
       disConnectQueued = false;
	   System.out.println( "Disconnected" );
       // This works, just starting the tread again doesn't for some reason
       // but re-creating it is OK. Need to see if I can figure out why a start
       // doesn't work, and investigate memory usage of this workaround.
       threadtask = null;
	} // disConnect

    public void writeBuffer( String dataBuffer ) {
       if (disConnectQueued) { disConnect(); }
       if (!(isConnected)) { isConnected = reConnect(); }
	   if (isConnected) {
          try { out.println( dataBuffer ); }
          catch (Exception e) {
                                  disConnectQueued = true;
                                  writeServerDownMsg();
                                  System.out.println( "IO Write error, starting disconnect" );
                               }
       } else {
          writeServerDownMsg();
       }
    }

	private void writeServerDownMsg() {
       // need to write a message to the list box
       myListModel.clear();
       updateTable( "----/--/--","---------------","--------------","IO","Alert Server is Unavailable",Color.red );
	}

   private void process_CONSOLE_response(String sDataVar) {
        /**
        This is only ever called internally !.
        Called by run() if the data block recived by the most recent read of the
        socket was an API response from the scheduler subsystem to a listall
        response.
        Whenever a valid data line is found/built calls updateTable to do the
        insertion of the new line into the main JList.
        */

        int i, i2, i3, iDataLen;
        String localDataBuf, testchars, dataLen;
        String hostName, alertKey, dateTime, headerTypeKey, restOfLine;
        Color colorToUse;
        // Take a local copy of the data line, as we need to
        // change it.
        localDataBuf = sDataVar;
        try {
           // There is a ^ at the end of each logical data line from the server
           while ((i2 = localDataBuf.indexOf('^')) > 5) { // 5 to catch bad data or even
                                                          // ourselves substringing only 
                                                          // to the ^ we search for.
              // BUG: trim does not remove leading spaces

              // Move a logical line to restOfLine
              restOfLine = localDataBuf.substring( 0, i2 );
              restOfLine.trim();

              // Get alert source time
              dateTime = restOfLine.substring(0,14);
              restOfLine = restOfLine.substring(15,restOfLine.length());
              restOfLine.trim();  // remove leading and trailing spaces

              // get alert source hostname
              hostName = restOfLine.substring(0,15);
              restOfLine = restOfLine.substring(16,restOfLine.length());
              restOfLine.trim();  // remove leading and trailing spaces

              // get alert key value
              alertKey = restOfLine.substring(0,14);
              restOfLine = restOfLine.substring(15,restOfLine.length());
              restOfLine.trim();  // remove leading and trailing spaces

              // Get the message type
              headerTypeKey = restOfLine.substring(0,2);
              // What type of alert (what colour do we need) */
              colorToUse = Color.red;  // default is red if no match found
              if (headerTypeKey.equals("AC")) {
                 colorToUse = Color.red;     // Critical
              }
              else if (headerTypeKey.equals("AK")) {
                 colorToUse = Color.yellow;  // Acknowledged, still an error
              }
              else if (headerTypeKey.equals("AW")) {
                 colorToUse = Color.pink;  // Warning message
              }
              else if (headerTypeKey.equals("IO")) {
                 colorToUse = Color.green;   // Info Only
              }
              else {
                 System.out.println( "Unknown line type (" + headerTypeKey + ") using RED" );
              }

              // And the rest of the logical line (the error text)
              restOfLine = restOfLine.substring(3,restOfLine.length());
              restOfLine.trim();  // remove leading and trailing spaces

              // The alert message text is the rest of the line
              if (alertKey.length() > 0) {
                  updateTable( dateTime, hostName, alertKey,
                               headerTypeKey, restOfLine, colorToUse );
              }  // if there is an alert key value
              else {
                 System.out.println( "No alert key found, line discarded" );
              }

              // Need to adjust the full buffer we are parsing as well so the data we
              // have just parsed over is removed from the buffer being processed.
              if ((i2+1) < localDataBuf.length()) {
                 localDataBuf = localDataBuf.substring( (i2+1), localDataBuf.length() );
                 localDataBuf.trim();
              }
           } // while
      } 
      catch (IndexOutOfBoundsException e2) {
          System.out.println( "IndexOutOfBounds, string left is..." );
          System.out.println( localDataBuf );
      }
  } // process_CONSOLE_response method

  private void updateTable( String aTime, String hostname, String aKey, 
                    String statusdata, String messagetext, Color colorToUse ) {
      /**
      This is only ever called internally !.
      Called by process_CONSOLE_response when a JList record needs
      to be updated or inserted in the table component this thread is feeding
      information to.
      Note: uses invokeLater to ensure it's done properly through event logic.
      The parameters passed are the values for the first three columns of
      the table.
      */
      // Because they are accessed within an inner loop the passed
      // parameters must be declared final.
      final String at;
      final String hn;
      final String ak;
      final String sd;
      final String mt;
      final Color  co;
      at = aTime;
      hn = hostname;
      ak = aKey;
      sd = statusdata;
      mt = messagetext;
      co = colorToUse;

      Runnable updateTest = new Runnable() {
          public void run() {
              final ListItem li = new ListItem( co, 
                          at + " " + hn + " " + ak + " " + sd + " " + mt );
              myListModel.addElement(li);
          }
      }; // Runable
      SwingUtilities.invokeLater(updateTest);
  } // updateTable

  // Java just doesn't have a routine to convert a number held in a string
  // to an integer, we need one so create one here. It will only support
  // the ASCII character set. I can't afford an MVS mainframe to test any
  // EBCDIC handling and don't intend to bother coding it.
  public int StringToInt( String numberInString ) {
     int totalVal = 0;
     char testByte = '0';
     if ((int)testByte == 48) { // ascii character table
        char[] zz = numberInString.toCharArray();
        totalVal = 0;  // just in case it's not initialised to zero
        for (int i = 0; i < numberInString.length(); i++)
        {
           totalVal = (totalVal * 10) + ((int)zz[i] - 48);
        }
     } else { // must be ebcdic character set, not implemented
        System.out.println( "EBCDIC Character Set is not supported");
        totalVal = 0;
     }
     return totalVal;
  } // StringToInt

} /* end of SocketObject */
