/*
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
*/

/* 
 *  clean_shutdown.c :
 *
 *  syntax: clean_shutdown port ipaddr"
 *
 * port is the port the server is listening on
 * ipaddr is the ipaddress of the server
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include "system_type.h"

#include "../common/alertserver_api_def.h"
#include "../common/shared_utils.c"

/* *****************************************************************************
 * init_alert_request_buffer:
 * Initialise the alert message buffer, set it to default to a critical
 * message.
 * NOTE: Not all fields are initialised here, only ones we must force reset
 *       that are not going to be rebuild in the message processing proc.
   ***************************************************************************** */
void init_alert_request_buffer( message_data_def * databuf ) {
   strncpy( databuf->msg_header.productid, "KILLER", ASRV_PRODUCT_ID_LEN );
   strncpy( databuf->msg_header.version, application_version, ASRV_PRODUCT_VERSION_LEN );
   gethostname( (char *)&databuf->msg_header.hostname, MAX_ASRV_HOSTNAME_LEN );
   strncpy( databuf->msg_header.unique_key, "N/A", MAX_ASRV_KEY_LEN );
   databuf->msg_header.messagetype[0] = 'L';  /* log */
   databuf->msg_header.messagetype[1] = 'O';  /* only */
   strncpy( databuf->msg_header.response_port, "0", 6 );  /* not implemeted yet */
   databuf->msg_header.expect_reply = 'N';    /* this NEVER wants a reply */
   databuf->datapersists = 'Y';    /* keep after we disconnectas we alsways disconnect  */
   set_datestamp( (char *)&databuf->msg_header.source_date_time );
   zero_data_field( (char *)&databuf->datastr, MAX_ASRV_REMOTEHOST_DATA_LEN );
   zero_data_field( (char *)&databuf->optdata, MAX_ASRV_OPT_DATA );
   number_to_string_with_null( (char *)&databuf->datastrlen, 0, MAX_ASRV_LEN_STR+1 );
   pad_header_fields( (common_header_def *)databuf );
   databuf->NL = '\n';
} /*init_alert_request_buffer */


/* *****************************************************************************
 * connect_to_server:
 * Return 1 if OK, 0 if sockptr needed to be set to null
   ***************************************************************************** */
int connect_to_server(int portnum, char *ipaddr, int *sockptr) {
   int z;
   char   workbuf[4096];
   struct sockaddr_in adr_srvr; /* AF_INET */
   int    len_inet;           /* length */

   /* create server socket address */
   memset(&adr_srvr,0,sizeof adr_srvr);
   adr_srvr.sin_family = AF_INET;
   adr_srvr.sin_port = htons(portnum);
   adr_srvr.sin_addr.s_addr = inet_addr(ipaddr);
#ifdef SOLARIS
   /* Solaris does not have a INADDR_NONE defined
	* but returns -1 for a bad address */
   if (adr_srvr.sin_addr.s_addr == -1 )
#else
   if (adr_srvr.sin_addr.s_addr == INADDR_NONE )
#endif
   {
      printf( "Improperly formatted network address, %s\n", ipaddr );
      return( 0 );  /* failed */
   }
   len_inet = sizeof adr_srvr;

   /* create a tcpip socket to use */
   *sockptr = socket(PF_INET,SOCK_STREAM,0);
   if (*sockptr == -1) {
      printf( "%s: %s\n", strerror(errno), "Unable to create TCPIP socket" );
	  return( 0 ); /* failed */ 
   }

   /* connect to the server */
   z = connect(*sockptr,&adr_srvr,len_inet);
   if (z == -1) {
      snprintf( workbuf, 4095, "connect(2), host %s nothing listening on port %d", ipaddr, portnum );
      printf( "%s: %s\n", strerror(errno), workbuf );
	  return( 0 ); /* failed */
   }

   return( 1 );  /* All OK */
} /* connect_to_server*/


/* *****************************************************************************
 * disconnect_from_server:
 * Use *sockptr to pass the socket to close.
   ***************************************************************************** */
void disconnect_from_server( int sockptr ) {
   /* close the socket and exit */
   close(sockptr);
   putchar('\n');
   return;
} /* disconnect_from_server */


/* *****************************************************************************
 * main:
   ***************************************************************************** */
int main(int argc, char **argv) {
   int socketid; /* socket */
   char *zptr;
   char *srvr_addr = NULL;
   int  srvr_port, z;
   message_data_def alert_message;

   /* check all required arguments are present, we need
	* port numer, ipaddress, severity, key, message text */
   if (argc < 2) {
      printf( "Required arguments are missing !\n" );
      printf( "Need port-num ip-address\n" );
	  printf( "Please read the manual !\n" );
      exit(1);
   }

   /* get the port number */
   srvr_port = atoi(argv[1]);
   if (srvr_port == 0) {
      printf( "Illegal port number: %s\n", argv[1] );
      exit( 1 );
   }
   /* get the ip address */
   srvr_addr = argv[2];

   /* Connect to the alert server */
   if (connect_to_server( srvr_port, srvr_addr, &socketid ) == 0) {
      printf( "Unable to connect to server.\n" );
      exit( 1 );
   }

   init_alert_request_buffer( (message_data_def *)&alert_message );

   z = write( socketid, &alert_message, sizeof(message_data_def) );
   if ( z != sizeof(message_data_def) ) {
      printf( "Unable to write alert buffer to %s\n", srvr_addr );
   }

   /* close the socket and exit */
   disconnect_from_server( socketid );
   exit(0);
} /* main */
