/*
#  ----------------------------------------------------------------------------
#  Start Copyright block
#
#  Centralised Alert Console Application, alert collection and forwarding tools
#  Copyright (C) 2003    Mark Dickinson,  All rights reserved.
# 
#  This software is not warrented for suitability in any installation
#  other than my own, use it at your own risk.
#
#  This version 1.0.1 release has no charge, and may be implemented
#  by you on as many servers as you see fit.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Specific terms and conditions for this software
#  ===============================================
#  This version 1.0.1 may not be redistributed for profit by any organisation.
#
#  BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
#  FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
#  OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
#  PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
#  OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
#  TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
#  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
#  REPAIR OR CORRECTION.
#
#  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
#  WILL ANY COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES INCLUDING
#  ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT
#  OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
#  TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
#  YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
#  PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGES.
#
#  You MAY NOT claim, copyright, or patent any applications
#  that use this code in whole or in part, nor any programs
#  that may interface with this application.
#
#  The author may at any time change the interfaces between the main
#  application server and the tools that communcate with it. If you 
#  write any inetrface applications of your own to work in conjunction
#  with this application you do so at your own risk, knowing that you
#  may need to rewrite your applications between versions.
#
# End Copyright block
#  ----------------------------------------------------------------------------
*/

/* 
 *  reload_alerts.c :
 *
 *  syntax: reload_alerts port ipaddr filename"
 *
 * port is the port the server is listening on
 * ipaddr is the ipaddress of the server
 * filename may be provided to override the default of 'todays' log file
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>

#include "system_type.h"

#include "../common/alertserver_api_def.h"
#include "../common/shared_utils.c"

/* Not reqd. at present, I've got the pointers sorted out now.
#define DEBUG_ON
*/

/* *****************************************************************************
 * init_alert_request_buffer:
 *
 * Initialise the alert message buffer fields that won't change.
 * NOTE: Not all fields are initialised here, only ones we must force reset
 *       that are not going to be rebuild in the message processing proc.
   ***************************************************************************** */
void init_alert_request_buffer( message_data_def * databuf ) {
   strncpy( databuf->msg_header.productid, "RELOADER", ASRV_PRODUCT_ID_LEN );
   strncpy( databuf->msg_header.version, application_version, ASRV_PRODUCT_VERSION_LEN );
   strncpy( databuf->msg_header.unique_key, "N/A", MAX_ASRV_KEY_LEN );
   strncpy( databuf->msg_header.response_port, "0", 6 );  /* not implemeted yet */
   databuf->msg_header.expect_reply = 'N';    /* this NEVER wants a reply */
   databuf->datapersists = 'Y';    /* keep after we disconnectas we alsways disconnect  */
   zero_data_field( (char *)&databuf->datastr, MAX_ASRV_REMOTEHOST_DATA_LEN );
   zero_data_field( (char *)&databuf->optdata, MAX_ASRV_OPT_DATA );
   number_to_string_with_null( (char *)&databuf->datastrlen, 0, MAX_ASRV_LEN_STR+1 );
   pad_header_fields( (common_header_def *)databuf );
   databuf->NL = '\n';
} /*init_alert_request_buffer */


/* *****************************************************************************
 * connect_to_server:
 *
 * Establish the connection to the alert server task.
 * Return 1 if OK, 0 if sockptr needed to be set to null
   ***************************************************************************** */
int connect_to_server(int portnum, char *ipaddr, int *sockptr) {
   int z;
   char   workbuf[4096];
   struct sockaddr_in adr_srvr; /* AF_INET */
   int    len_inet;           /* length */

   /* create server socket address */
   memset(&adr_srvr,0,sizeof adr_srvr);
   adr_srvr.sin_family = AF_INET;
   adr_srvr.sin_port = htons(portnum);
   adr_srvr.sin_addr.s_addr = inet_addr(ipaddr);
#ifdef SOLARIS
   /* Solaris does not have a INADDR_NONE defined
	* but returns -1 for a bad address */
   if (adr_srvr.sin_addr.s_addr == -1 )
#else
   if (adr_srvr.sin_addr.s_addr == INADDR_NONE )
#endif
   {
      printf( "Improperly formatted network address, %s\n", ipaddr );
      return( 0 );  /* failed */
   }
   len_inet = sizeof adr_srvr;

   /* create a tcpip socket to use */
   *sockptr = socket(PF_INET,SOCK_STREAM,0);
   if (*sockptr == -1) {
      printf( "%s: %s\n", strerror(errno), "Unable to create TCPIP socket" );
	  return( 0 ); /* failed */ 
   }

   /* connect to the server */
   z = connect(*sockptr,&adr_srvr,len_inet);
   if (z == -1) {
      snprintf( workbuf, 4095, "connect(2), host %s nothing listening on port %d", ipaddr, portnum );
      printf( "%s: %s\n", strerror(errno), workbuf );
	  return( 0 ); /* failed */
   }

   return( 1 );  /* All OK */
} /* connect_to_server*/


/* *****************************************************************************
 * disconnect_from_server:
 *
 * Disconnect/close the session to the alert server task.
 * Use *sockptr to pass the socket to close.
   ***************************************************************************** */
void disconnect_from_server( int sockptr ) {
   /* close the socket and exit */
   close(sockptr);
   putchar('\n');
   return;
} /* disconnect_from_server */


/* *****************************************************************************
 * log_timespamp_of_now:
 *
 * create a timestamp string of the time now in the same format as that used
 * to write the log messages by the server task.
 * Used to ensure that if we are reloading alerts from the current days file
 * for some reason, we stop at the current time (so we don't process the log
 * records that are written by ourself after the start eof as we re-add the
 * new ones and keep extending the eof).
   ***************************************************************************** */
void log_timestamp_of_now( char *buffer, int buflen ) {
   time_t time_now;
   char *p2, time_now_text[50];
   struct tm time_var;
   struct tm *time_var_ptr;
   time_now = time(0);
   time_var_ptr = &time_var;
   time_var_ptr = localtime(&time_now);
   snprintf( time_now_text, 49, "%s", ctime(&time_now) );
   p2 = (char *)&time_now_text;
   p2 = p2 + (strlen(time_now_text) - 1);
   *p2 = '\0';
   strncpy( buffer, time_now_text, buflen );
} /* log_timestamp_of_now */


/* *****************************************************************************
 * process_single_log_line:
 *
 * pasrse a single line from the log file and pull out the fields we need to
 * build a new alert message, formatting them into the alert record passed.
 * We ONLY intend to re-load real alerts or alert cancels, we will not bother
 * with warning, informational or log only events.
 *
 * Returns 0 if no action to be taken for the line, 1 if the alert buffer
 * needs to be sent to the server.
   ***************************************************************************** */
int process_single_log_line( message_data_def *alert_message, char *dataline ) {
   char *ptr, *ptr3;
   int resultvalue;

   /* only reload alert raise, ack or delete requests, ignore warnings and
	* logonly events */
   zero_data_field( (char *)&alert_message->datastr, MAX_ASRV_REMOTEHOST_DATA_LEN );
   zero_data_field( (char *)&alert_message->optdata, MAX_ASRV_OPT_DATA );

   ptr = dataline;  /* refer to the dataline */

   ptr3 = ptr + 62; /* alert type code */
#ifdef DEBUG_ON
   printf("DEBUG: Checking alert type %s\n", ptr3 );
#endif
   alert_message->msg_header.messagetype[0] = *ptr3;
   ptr3++;
   alert_message->msg_header.messagetype[1] = *ptr3;
   if (memcmp(alert_message->msg_header.messagetype,"AD",2) == 0) {
      /* AD becomes AK so server processes it from us rather than a console */
      alert_message->msg_header.messagetype[1] = 'K'; /* AD becomes AK */
   }
   if ( (memcmp(alert_message->msg_header.messagetype,"AC",2) == 0) ||
        (memcmp(alert_message->msg_header.messagetype,"AK",2) == 0) ||
        (memcmp(alert_message->msg_header.messagetype,"KA",2) == 0) ) {
       ptr3 = ptr + 31; /* originating host offset */
       strncpy( alert_message->msg_header.hostname, ptr3, MAX_ASRV_HOSTNAME_LEN );
       alert_message->msg_header.hostname[MAX_ASRV_HOSTNAME_LEN] = '\0';
       ptr3 = ptr + 47; /* date offset */
       strncpy(alert_message->msg_header.source_date_time, ptr3, 14 );
       alert_message->msg_header.source_date_time[14] = '\0';
       ptr3 = ptr + 65; /* alert key value */
       strncpy( alert_message->msg_header.unique_key, ptr3, MAX_ASRV_KEY_LEN );
       ptr3 = ptr + 82; /* alert text value */
       strncpy( alert_message->datastr, ptr3, MAX_ASRV_REMOTEHOST_DATA_LEN );
       alert_message->datastr[MAX_ASRV_REMOTEHOST_DATA_LEN] = '\0'; /* ensure terminated with a null */
       number_to_string_with_null( (char *)&alert_message->datastrlen, strlen(alert_message->datastr), MAX_ASRV_LEN_STR+1 );
       alert_message->datastrlen[MAX_ASRV_LEN_STR] = '\0'; /* wasn't null terminated, is now */
       resultvalue = 1;  /* need to write it */
   }
   else {
       resultvalue = 0;  /* no action with this line */
   }

   return( resultvalue ); /* disabled */
} /* process_single_log_line */


/* *****************************************************************************
 * process_alert_log:
 *
 * open the alert log and scan through it until eof (for a non-today file) or
 * until we decide we are done when we reach messages with timestamps after
 * the current time (in which case its todays log file we are scanning and
 * we have reached the log records we caused to be written while re-loading
 * the alerts.
   ***************************************************************************** */
int process_alert_log( char *filename, int socketid ) {
#define DATABUF_SIZE 1024
   int z, bytes_to_read, data_used, data_left, sockz, time_in_future;
   message_data_def alert_message;
   FILE *logfile;
   char databuffer[DATABUF_SIZE+1];
   char *ptr, *ptr_end_of_last_line, *ptr3, *ptr_eoll;
   char time_to_stop[50];

   if ( (logfile=fopen(filename,"r")) == NULL ) {
       perror( "Unable to open file" );
	   printf( "Error opening %s\n", filename );
	   return( 1 );
   }

   init_alert_request_buffer( &alert_message );
   time_in_future = 0;
   log_timestamp_of_now( time_to_stop, 50 ); /* last possible original alert */

   /* We get multiple lines with each read so we must parse them out */
   ptr = (char *)&databuffer;
   bytes_to_read = DATABUF_SIZE;
   while ( (!(feof(logfile))) && (!(ferror(logfile))) && (time_in_future == 0) ) {
      z = fread( ptr, 1, bytes_to_read, logfile );
	  if (z > 0) {
#ifdef DEBUG_ON
         printf("DEBUG: read %d bytes\n", z );
#endif
         databuffer[DATABUF_SIZE] = '\0';
         ptr_end_of_last_line = ptr + z;   /* find the last newline in the buffer */
         ptr = (char *)&databuffer;  /* with end of buffer set move ptr back to start of buffer */
         while ((*ptr_end_of_last_line != '\n') && (ptr_end_of_last_line > ptr)) { ptr_end_of_last_line--; }
#ifdef DEBUG_ON
         printf( "DEBUG: Full buffer is:%s\n", ptr );
         printf("DEBUG: data not to be processed in this pass: %s\n", ptr_end_of_last_line );
#endif
         while (ptr < ptr_end_of_last_line) {
             ptr_eoll = ptr;   /* find the end of the logical line */
             while ((*ptr_eoll != '\n') && (ptr_eoll < ptr_end_of_last_line)) { ptr_eoll++; }
             *ptr_eoll = '\0';  /* terminate for our copies */
#ifdef DEBUG_ON
		     printf( "GOT: %s\n", ptr );
#endif
                if (process_single_log_line( &alert_message, ptr ) != 0) {
                   /* put this check inside here, I had it outside and it
					* triggered on partial buffers remaining */
                   if (memcmp(ptr,time_to_stop,19) >= 0) {
                      time_in_future = 1;
#ifdef DEBUG_ON
                      printf( "DEBUG: Stopping load as log message time\n %s\n is after our stop time of %s\n", ptr, time_to_stop );
#endif
                   }
                   else {
                       /* needs to be written */
                       sockz = write( socketid, &alert_message, sizeof(message_data_def) );
                       if ( sockz != sizeof(message_data_def) ) {
                          printf( "Unable to write alert buffer to server\n" );
                       }
                   } /* if message in in past check */
                }
             ptr = ptr_eoll;
             ptr++;
         }  /* while ptr not at end of last logical line */
		 /* now move any remaining data to the start of the buffer */
         ptr = (char *)&databuffer;
         ptr_end_of_last_line++; /* skip the nl we are sitting on */
         data_used = (ptr_end_of_last_line - ptr);
         data_left = z - data_used;
         if (data_left > 0) {
#ifdef DEBUG_ON
             printf("DEBUG: data read = %d, data used = %d, data left = %d\n", z, data_used, data_left );
             printf("DEBUG: preserving %d bytes of buffer: %s\n", data_left, ptr_end_of_last_line );
#endif
             strncpy( ptr, ptr_end_of_last_line, data_left );
             ptr = ptr + data_left; /* adjust ptr to end of data for the next read */
             bytes_to_read = DATABUF_SIZE - data_left; /* adjust buffer size we can read into */
         }
		 else {
#ifdef DEBUG_ON
            printf("DEBUG: no data to preserve\n" );
#endif
            ptr = (char *)&databuffer;
            bytes_to_read = DATABUF_SIZE;
         }
	  } /* if z > 0 */
      else { /* must be eof, adjust pointers to be safe */
         ptr = (char *)&databuffer;
         bytes_to_read = DATABUF_SIZE;
      }
   } /* while not eof and not error & not time to stop */

   fclose( logfile );
   return( 0 );
} /* process_alert_log */


/* *****************************************************************************
 * main:
   ***************************************************************************** */
int main(int argc, char **argv) {
   int command_result;
   int socketid; /* socket */
   char *zptr;
   char *srvr_addr = NULL;
   int  srvr_port;
   char workbuffer[100];
   char filename_to_use[255];

   printf("Centralised Alert Console Application, alert collection and forwarding tools\n");
   printf("Copyright (C) 2003    Mark Dickinson,  All rights reserved.\n");
   printf("Utility: reload_alerts\n \n");
   printf( "This software is released free with ABSOLUTELY NO WARRANTY under the terms\n");
   printf("of the GPL license, refer to the copy of the GPL license provided with this\n");
   printf("application for more details\n \n" );

   /* check all required arguments are present, we need
	* port numer, ipaddress, and filename */
   if (argc < 3) {
      printf( "Required arguments are missing !\n" );
      printf( "Need port-num ip-address filename\n" );
	  printf( "Please read the manual !\n" );
      exit(1);
   }

   /* get the port number */
   srvr_port = atoi(argv[1]);
   if (srvr_port == 0) {
      printf( "Illegal port number: %s\n", argv[1] );
      exit( 1 );
   }
   /* get the ip address */
   srvr_addr = argv[2];

   /* get the alert log filename to read */
   strncpy( filename_to_use, argv[3], 254 );
   filename_to_use[254] = '\0';
#ifdef DEBUG_ON
   printf( "DEBUG: Using %d %s %s\n", srvr_port, srvr_addr, filename_to_use );
#endif

   /* Connect to the alert server */
   if ((command_result = connect_to_server( srvr_port, srvr_addr, &socketid )) == 0) {
      printf( "Unable to connect to server.\n" );
      exit( 1 );
   }

   /* Lets go and process the file now */
   command_result = process_alert_log( (char *)&filename_to_use, socketid );

   /* close the socket and exit */
   disconnect_from_server( socketid );
   exit(command_result);
} /* main */
